/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  brushed.c - Definition file for functions related to brushed motion control
 *  DRV87xx_8106-Q1EVM_FIRMWARE
 *  10/25/2021
 *  Murugavel Raju
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include <brushed.h>
#include <drv8x06.h>
#include <SPI.h>
#include "msp430f2617.h"

/********************************************************************************
 * MACROS
 *******************************************************************************/
#pragma DATA_SECTION(gBrushed_Obj,".stpVarSection");
volatile brushed_Driving_Obj_t gBrushed_Obj;

/*****************************************************************************
 * Define Global Variables in brushed.h
 *****************************************************************************/
uint16_t SOSense = 2048;
uint16_t SOSense2 = 2048;
extern int16_t SOSense_AREF_Adjusted;
extern int16_t SOSense2_AREF_Adjusted;

extern int pre_ph;
extern int pre_ph2;
extern int timerUpdate;
/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
uint8_t getPWMMode()
{
    uint8_t temp;

    if (!DRV871x)                                                            // If device is DRV8106S-Q1, DRV870xS-Q1
    {
        uint8_t temp1 = 0x60 & (uint8_t) SPI_readRegister(BRG_CTRL);
        temp = temp1 >> 5;
    }

    if (gDRV8x06_Obj.deviceID == 7 || gDRV8x06_Obj.deviceID == 8)            // If device is DRV871xS-Q1
    {
        uint8_t temp2 = 0x30 & (uint8_t) SPI_readRegister(IC_CTRL1_DRV871XX);
        temp = temp2 >> 4;
    }

    return temp;
}

void drv_setCtrlMode()
{
    // Update control mode
    if (gBrushed_Obj.pwmMode == 0)                                           // Independent Half-Bridges Mode
    {
        if (DRV871x)                                                         // If device is DRV8714S/H-Q1 or DRV8718S-Q1
        {
            gBrushed_Obj.ph = 0;
            gBrushed_Obj.ph2 = 0;
            P4OUT &= ~(IN1_EN + IN2_PH + IN3_EN + IN4_PH);                   // set to low
            P4SEL &= ~(IN1_EN + IN2_PH + IN3_EN + IN4_PH);                   // Set IN1 and IN2 to GPIO outputs

            TBCCTL2 |= OUTMOD_7;                                             // IN1 setup as reset/set timer PWM output
            TBCCTL3 |= OUTMOD_7;                                             // IN2 setup as reset/set timer PWM output
            TBCCTL5 |= OUTMOD_7;                                             // IN3 setup as reset/set timer PWM output
            TBCCTL6 |= OUTMOD_7;                                             // IN4 setup as reset/set timer PWM output
            TBCCR2 = 0;
            TBCCR3 = 0;
            TBCCR5 = 0;
            TBCCR6 = 0;
            P4SEL |= IN1_EN + IN2_PH + IN3_EN + IN4_PH;                      // IN1, IN2, IN3 and IN4 setup as timer PWM outputs
        }
        else                                                                 // Device is DRV8106S/H-Q1, DRV870xS/H-Q1
        {
            P1DIR |= nHIZ1 | nHIZ2;                                          // Enable nHIZx as an output

            gBrushed_Obj.ph = 0;
            P4OUT &= ~(IN1_EN + IN2_PH + IN3_EN + IN4_PH);                   // set to low
            P4SEL &= ~(IN1_EN + IN2_PH + IN3_EN + IN4_PH);                   // Set IN1, IN2, IN3 and IN4 to GPIO outputs

            TBCCTL5 &= ~OUTMOD_7;
            TBCCTL6 &= ~OUTMOD_7;
            TBCCR5 = 0;
            TBCCR6 = 0;
            TBCCTL2 |= OUTMOD_7;                                             // IN1 setup as reset/set timer PWM output
            TBCCTL3 |= OUTMOD_7;                                             // IN2 setup as reset/set timer PWM output
            TBCCR2 = 0;
            TBCCR3 = 0;
            P4SEL |= IN1_EN + IN2_PH;                                        // IN1 and IN2 setup as timer PWM outputs
        }
    }
    else if (gBrushed_Obj.pwmMode == 1)                                      // Phase Enable bridge control Full-Bridge Mode
    {
        if (!DRV871x)                                                        // If device is DRV8106S/H-Q1, DRV870xS/H-Q1
        {
            P1DIR &= ~nHIZ1;                                                 // nHIZx direction as input to make it High Z
            P1DIR &= ~nHIZ2;                                                 // nHIZx direction as input to make it High Z
        }

        TBCCTL3 &= ~OUTMOD_7;                                                // IN2 timer PWM off
        TBCCTL2 |= OUTMOD_7;                                                 // Reset/set PWM output mode for IN1_EN
        TBCCR2 = 0;
        TBCCR3 = 0;

        pre_ph = gBrushed_Obj.ph;                                            // If ph is 0, pre_ph becomes 0

        if (gBrushed_Obj.ph)                                                 // Forward direction is selected
            P4OUT |= IN2_PH;
        else if (!gBrushed_Obj.ph)                                           // Reverse direction is selected
            P4OUT &= ~IN2_PH;

        P4SEL &= ~IN2_PH;                                                    // Set IN2 to GPIO output

        if (gDRV8x06_Obj.deviceID == 8 || gDRV8x06_Obj.deviceID == 9)        // DRV8714S/H-Q1
        {
            TBCCTL6 &= ~OUTMOD_7;                                            // IN4 timer PWM off
            TBCCTL5 |= OUTMOD_7;                                             // Reset/set PWM output mode for IN3_EN
            TBCCR5 = 0;
            TBCCR6 = 0;

            pre_ph2 = gBrushed_Obj.ph2;                                      // if ph2 is 0, pre_ph2 becomes 0

            if (gBrushed_Obj.ph2)                                            // Forward direction is selected
                P4OUT |= IN4_PH;
            else if (!gBrushed_Obj.ph2)                                      // Reverse direction is selected
                P4OUT &= ~IN4_PH;

            P4SEL &= ~IN4_PH;                                                // Set IN4 to GPIO output
        }
    }
    else if (gBrushed_Obj.pwmMode == 2)                                      // PWM bridge control Full-Bridge Mode
    {
        if (!DRV871x)                                                        // If device is DRV8106S/H-Q1, DRV870xS/H-Q1
        {
            P1DIR &= ~nHIZ1;                                                 // nHIZx direction as input to make it High Z
            P1DIR &= ~nHIZ2;                                                 // nHIZx direction as input to make it High Z
        }

        gBrushed_Obj.ph = 0;
        P4OUT &= ~(IN1_EN + IN2_PH);                                         // set to low
        P4SEL &= ~(IN1_EN + IN2_PH);                                         // Set IN1 and IN2 to GPIO outputs

        TBCCTL2 |= OUTMOD_7;
        TBCCTL3 |= OUTMOD_7;
        TBCCR2 = 0;
        TBCCR3 = 0;

        if (gDRV8x06_Obj.deviceID == 8 || gDRV8x06_Obj.deviceID == 9)        // DRV8714S/H-Q1
        {
            gBrushed_Obj.ph2 = 0;
            P4OUT &= ~(IN3_EN + IN4_PH);                                     // set to low
            P4SEL &= ~(IN3_EN + IN4_PH);                                     // Set IN1, IN2, IN3 and IN4 to GPIO outputs

            TBCCTL5 |= OUTMOD_7;
            TBCCTL6 |= OUTMOD_7;
            TBCCR5 = 0;
            TBCCR6 = 0;
        }
    }
}

void setDefault_GUI_DRV_Vals()
{
    gDRV8x06_Obj.nSleep = 1;                                                 // Set GUI variable to device is awake
    if (gDRV8x06_Obj.nSleep)
    {
    set_nSLEEP_hi;
    __delay_cycles(20000);
    nSLEEP_is_hi = 1;
    }
    else
    {
        set_nSLEEP_lo;
        __delay_cycles(20000);
        nSLEEP_is_hi = 0;
    }

    if (SVariant)                                                            // For all S variants
    {
        SPI_Init();
        reg_Init();

        if (gDRV8x06_Obj.deviceID == 3 || DRV8718S)
            gBrushed_Obj.pwmMode = 0;                                        // DRV8106S-Q1 or DRV8718S-Q1 Half-Bridge devices
        else
            gBrushed_Obj.pwmMode = getPWMMode();                             // PH/EN except for DRV8106S-Q1 and DRV8718S-Q1
    }
    else if (gDRV8x06_Obj.deviceID == 2 || gDRV8x06_Obj.deviceID == 6
            || gDRV8x06_Obj.deviceID == 9)                                   // DRV870xH-Q1 and DRV8714H-Q1 Full-Bridge devices
    {
        gBrushed_Obj.pwmMode = 1;                                            // PH/EN by default
    }
    else if (gDRV8x06_Obj.deviceID == 4)                                     //  DRV8106H-Q1EVM
        gBrushed_Obj.pwmMode = 0;

    if (!DRV871x)                                                            // If device is DRV8106S/H-Q1, DRV870xS/H-Q1
    {
        gBrushed_Obj.nhiz1 = 0;
        set_nHIZ1_lo;

        gBrushed_Obj.nhiz2 = 0;
        set_nHIZ2_lo;

        gDRV8x06_Obj.drvOFF = 1;                                             // outputs are off
        set_DRVOFF_hi;
    }

    if (DRV871x)                                                             // If device is DRV871xS/H-Q1
    {
        gBrushed_Obj.in3En = 0;
        set_IN3_EN_lo;

        gBrushed_Obj.in4Ph = 0;
        set_IN4_PH_lo;

        gBrushed_Obj.ph2 = 0;                                                // reverse

        gBrushed_Obj.soReadComplete2 = 0;

        // Ramp controller init for IN3
        gBrushed_Obj.rampInput3.rampDelayMax = 255;
        gBrushed_Obj.rampInput3.rampDelayCount = 0;
        gBrushed_Obj.rampInput3.setpointValue = 0;

        // Ramp controller init for IN4
        gBrushed_Obj.rampInput4.rampDelayMax = 255;
        gBrushed_Obj.rampInput4.rampDelayCount = 0;
        gBrushed_Obj.rampInput4.setpointValue = 0;

        gBrushed_Obj.ADC12read2 = SOSense2_DEFAULT;
    }

    gBrushed_Obj.ph = 0;                                                     // reverse

    gBrushed_Obj.in1En = 0;
    set_IN1_EN_lo;

    gBrushed_Obj.in2Ph = 0;
    set_IN2_PH_lo;

    gBrushed_Obj.startMotor = 0;                                             // start motor button is off
    gBrushed_Obj.pwmState = 0;                                               // PWM can be started.
    gBrushed_Obj.hBMotorConn = 2;
    gBrushed_Obj.soReadComplete = 0;

    // Ramp controller init for IN1
    gBrushed_Obj.rampInput1.rampDelayMax = 255;
    gBrushed_Obj.rampInput1.rampDelayCount = 0;
    gBrushed_Obj.rampInput1.setpointValue = 0;

    // Ramp controller init for IN2
    gBrushed_Obj.rampInput2.rampDelayMax = 255;
    gBrushed_Obj.rampInput2.rampDelayCount = 0;
    gBrushed_Obj.rampInput2.setpointValue = 0;

    gDRV8x06_Obj.landingPage = 1;
    gDRV8x06_Obj.firmwareVersion = FIRMWARE_VERSION;
    gBrushed_Obj.ADC12read = SOSense_DEFAULT;                                // Display SO value

    // Setup IC Revision Identification for GUI
    gDRV8x06_Obj.revID = 16;                                                 // 1000b for 2p0
}

// reads GUI variables and updates the GPIOs
void updateDeviceControls()
{
    if (DRV871x)                                                             // If device is DRV871xS/H-Q1
    {
        if (gDRV8x06_Obj.deviceID == 9)                                      // DRV8714H variant has only nfault reporting, no drvoff
        {
            if (read_DRVOFF_nFLT_INT_pin == 0)
                // nFault has tripped
                gDRV8x06_Obj.nFault = 0;                                     // Fault active low
            else
                gDRV8x06_Obj.nFault = 1;
        }
        else if (((gDeviceSpiReg.ic_ctrl2_reg >> 6) & 1) == 0)               // If setup as DRVOFF in SPI settings
        {
            P1DIR |= DRVOFF_nFLT_INT;                                        // Make this GPIO an output for DRVOFF function
            set_LED_DIS_lo;

            if (gDRV8x06_Obj.drvOFF)                                         // Outputs are turned off in the GUI
                set_DRVOFF_nFLT_INT_hi;
            else
                set_DRVOFF_nFLT_INT_lo;
        }
        else                                                                 // setup as nFAULT
        {
            P1DIR &= ~DRVOFF_nFLT_INT;                                       // make this GPIO an input for nFAULT reading
            set_LED_DIS_HIZ;                                                 // enable LED power by HiZ LED_DIS GPIO

            if (read_DRVOFF_nFLT_INT_pin == 0)                               // nFault has tripped
                gDRV8x06_Obj.nFault = 0;                                     // active low, set GUI variable to display nFault
            else
                gDRV8x06_Obj.nFault = 1;
        }
    }
    else                                                                     // Else if device is DRV8106S/H-Q1, DRV8705/06S/H-Q1
    {
        if (read_nFAULT_INT_pin == 0)                                        // nFault has tripped
            gDRV8x06_Obj.nFault = 0;
        else
            gDRV8x06_Obj.nFault = 1;

        if (gDRV8x06_Obj.drvOFF)
            // outputs are off
            set_DRVOFF_hi;
        else
            set_DRVOFF_lo;

        if (gBrushed_Obj.pwmMode == 0)
        {
            if (gBrushed_Obj.nhiz1)                                          // on
                set_nHIZ1_hi;
            else
                set_nHIZ1_lo;

            if (gBrushed_Obj.nhiz2)                                          // on
                set_nHIZ2_hi;
            else
                set_nHIZ2_lo;
        }
    }

    if (gDRV8x06_Obj.nSleep && !nSLEEP_is_hi)                                // If GUI App set to device awake and device was in sleep mode
        {
        set_nSLEEP_hi;
        __delay_cycles(20000);
        nSLEEP_is_hi = 1;
        }
    else if (!gDRV8x06_Obj.nSleep && nSLEEP_is_hi)
        {
        set_nSLEEP_lo;
        nSLEEP_is_hi = 0;
        }
}

void updateSOSenseAREF()
{
    //Adjust default SOSense for AREF divider, HW device variant has only AREF/2
    if (DRV871x)                           // Device is DRV871xS/H-Q1
    {
        SOSense_AREF_Adjusted = SOSense_DEFAULT;
        SOSense2_AREF_Adjusted = SOSense2_DEFAULT;
        if (SVariant)
        {
            if (gDeviceSpiReg.csa_ctrl1_reg & BIT5)
                SOSense_AREF_Adjusted = SOSense_DIV8_DEFAULT;
            if (gDeviceSpiReg.csa_ctrl1_reg & BIT2)
                SOSense2_AREF_Adjusted = SOSense2_DIV8_DEFAULT;
        }
    }
    else                                   // Device is DRV8106S/H-Q1 or DRV870xS/H-Q1
    {
        SOSense_AREF_Adjusted = SOSense_DEFAULT;
        if (SVariant)
        {
            if (gDeviceSpiReg.csa_ctrl1_reg & BIT2)
                SOSense_AREF_Adjusted = SOSense_DIV8_DEFAULT;
        }
    } // End adjust default SOSense for AREF divider

    //Update GUI display with default SOSense values
    gBrushed_Obj.ADC12read = SOSense_AREF_Adjusted;
    gBrushed_Obj.ADC12read2 = SOSense2_AREF_Adjusted;
}

void updateSOReading()
{
    //calculate average of collected samples
    volatile uint8_t loop;
    volatile unsigned long SOSense_sum = 0;
    volatile unsigned long SOSense2_sum = 0;
    extern unsigned int SOSense_Raw[];
    extern unsigned int SOSense2_Raw[];
    extern unsigned char SamplesCollected;

    if (SamplesCollected)
    {
        loop = 0;

        while (loop != NumSamples)
        {
            SOSense_sum += SOSense_Raw[loop];

            if (DRV871x)
            {
                SOSense2_sum += SOSense2_Raw[loop];
            }

            loop++;
        }

        SOSense = SOSense_sum/NumSamples;                                    // Divide for average calculation
        SOSense2 = SOSense2_sum/NumSamples;                                  // Divide for average calculation
        SamplesCollected = 0;                                                // Flag to manage 128 samples collection for average calculation
        ADC12CTL0 |= ENC;                                                    // Enable ADC conversions
    }
}

void SetVariantFlags ()
{
    if (gDRV8x06_Obj.deviceID == 1 || gDRV8x06_Obj.deviceID == 3
      || gDRV8x06_Obj.deviceID == 5 || gDRV8x06_Obj.deviceID == 7
      || gDRV8x06_Obj.deviceID == 8)           // S variant
        SVariant = 1;
    else
        SVariant = 0;                          // H variant

    if (gDRV8x06_Obj.deviceID == 7 || gDRV8x06_Obj.deviceID == 8
      || gDRV8x06_Obj.deviceID == 9 )
    {
        DRV871x = 1;                           // DRV8714S/H-Q1 or DRV8718S-Q1
        if (gDRV8x06_Obj.deviceID == 7)
            DRV8718S = 1;                      // DRV8718S-Q1
        else
            DRV8718S = 0;                      // DRV8714S/H-Q1
    }
    else
        DRV871x = 0;                           // DRV8106S/H-Q1, DRV8705/06S/H-Q1
}

void updateSVariants()
{
    update_Regs();
    gBrushed_Obj.pwmMode = getPWMMode();
}

void updatePWM()
{
    if (gBrushed_Obj.pwmMode == 0)                                           // Independent Half-Bridge Mode
    {
        if (gBrushed_Obj.in1En == 100 && TBCCR2 == 800)
        {
            if (TBCCR2 == 800)
            {
                P4OUT |= IN1_EN;                                             // set the output to 100%
                P4SEL &= ~IN1_EN;                                            // Set IN1 to GPIO output
            }
        }
        else
        {
            P4SEL |= IN1_EN;                                                 // Set IN1 to timer PWM output
            drv_IHBRampControl();
            TBCCR2 = gBrushed_Obj.rampInput1.setpointValue * 8;
        }

        if (gDRV8x06_Obj.deviceID == 3 || gDRV8x06_Obj.deviceID == 4)        // If device is DRV8106S/H-Q1 only one half-bridge
        {
            P4SEL &= ~IN2_PH;                                                // Set IN2 to GPIO output
        }
        else
        {
            if (gBrushed_Obj.in2Ph == 100 && TBCCR3 == 800)
            {
                if (TBCCR3 == 800)
                {
                    P4OUT |= IN2_PH;                                         // set the output to 100%
                    P4SEL &= ~IN2_PH;                                        // Set IN2 to GPIO output
                }
            }
            else
            {
                P4SEL |= IN2_PH;                                             // Set IN2 to timer PWM output
                drv_IHBRampControl();
                TBCCR3 = gBrushed_Obj.rampInput2.setpointValue * 8;
            }
        }

    if (DRV871x)                                                             // If device is DRV871xS/H-Q1 two additional PWMs IN3 and IN4
    {
        if (gBrushed_Obj.in3En == 100 && TBCCR5 == 800)
        {
            if (TBCCR5 == 800)
            {
                P4OUT |= IN3_EN;                                             // set the output to 100%
                P4SEL &= ~IN3_EN;                                            // Set IN3 to GPIO output
            }
        }
        else
        {
            P4SEL |= IN3_EN;                                                 // Set IN3 to timer PWM output
            drv_IHBRampControl();
            TBCCR5 = gBrushed_Obj.rampInput3.setpointValue * 8;
        }

            if (gBrushed_Obj.in4Ph == 100 && TBCCR6 == 800)
            {
                if (TBCCR6 == 800)
                {
                    P4OUT |= IN4_PH;                                         // set the output to 100%
                    P4SEL &= ~IN4_PH;                                        // Set IN4 to GPIO output
                }
            }
            else
            {
                P4SEL |= IN4_PH;                                             // Set IN4 to timer PWM output
                drv_IHBRampControl();
                TBCCR6 = gBrushed_Obj.rampInput4.setpointValue * 8;
            }
        }
    }
    else if (gBrushed_Obj.pwmMode == 1)                                      // Phase Enable Full-Bridge Mode
    {
        if (gBrushed_Obj.in1En == 100 && TBCCR2 == 800)                      // GUI and FW have reached 100% PWM
        {
            if (gBrushed_Obj.ph && !pre_ph)                                  //Direction change, forward
                TBCCR2 = 799;
            else if (!gBrushed_Obj.ph && pre_ph)                             //Direction change, reverse
                TBCCR2 = 799;
            else                                                             // no direction change at 100% PWM, just make sure output is GPIO 100%
            {
                P4OUT |= IN1_EN;                                             // set the output to 100%
                P4SEL &= ~IN1_EN;                                            // Set IN1 to GPIO output
            }
        }
        else
        {
            P4SEL |= IN1_EN;                                                 // Set IN1 to timer PWM output
            drv_PH_EN_RampControl();
            TBCCR2 = gBrushed_Obj.rampInput1.setpointValue * 8;
        }
        if (gDRV8x06_Obj.deviceID == 8 || gDRV8x06_Obj.deviceID == 9)        // If device is DRV8714S/H-Q1
        {
            if (gBrushed_Obj.in3En == 100 && TBCCR5 == 800)                  // GUI and FW have reached 100% PWM
            {
                if (gBrushed_Obj.ph2 && !pre_ph2)                            //Direction change, forward
                    TBCCR5 = 799;
                else if (!gBrushed_Obj.ph2 && pre_ph2)                       //Direction change, reverse
                    TBCCR5 = 799;
                else                                                         // no direction change at 100% PWM, just make sure output is GPIO 100%
                {
                    P4OUT |= IN3_EN;                                         // set the output to 100%
                    P4SEL &= ~IN3_EN;                                        // Set IN3 to GPIO output
                }
            }
            else
            {
                P4SEL |= IN3_EN;                                             // Set IN3 to timer PWM output
                drv_PH_EN_RampControl();
                TBCCR5 = gBrushed_Obj.rampInput3.setpointValue * 8;
            }
        }
    }
    else if (gBrushed_Obj.pwmMode == 2)                                      // PWM Full-Bridge Mode
    {
        if (gBrushed_Obj.in1En == 100 && TBCCR2 == 0)
        {
            P4OUT &= ~IN1_EN;                                                // set the output to 0%
            P4SEL &= ~IN1_EN;                                                // Set IN1 to GPIO output
        }
        if (gBrushed_Obj.in1En == 0 && TBCCR2 == 800)
        {
            P4OUT |= IN1_EN;                                                 // set the output to 100%
            P4SEL &= ~IN1_EN;                                                // Set IN1 to GPIO output
        }
        else
        {
            drv_RampCTRL(&gBrushed_Obj.rampInput1, gBrushed_Obj.in1En);      //Ramp controller for en_in1
            TBCCR2 = 800 - gBrushed_Obj.rampInput1.setpointValue * 8;
            P4SEL |= IN1_EN;                                                 // Set IN1 to timer PWM output
        }

        if (gBrushed_Obj.in2Ph == 100 && TBCCR3 == 0)
        {
            P4OUT &= ~IN2_PH;                                                // set the output to 0%
            P4SEL &= ~IN2_PH;                                                // Set IN2 to GPIO output
        }
        if (gBrushed_Obj.in2Ph == 0 && TBCCR3 == 800)
        {
            P4OUT |= IN2_PH;                                                 // set the output to 100%
            P4SEL &= ~IN2_PH;                                                // Set IN2 to GPIO output
        }
        else
        {
            drv_RampCTRL(&gBrushed_Obj.rampInput2, gBrushed_Obj.in2Ph);      //Ramp controller for ph_in2
            TBCCR3 = 800 - gBrushed_Obj.rampInput2.setpointValue * 8;
            P4SEL |= IN2_PH;                                                 // Set IN2 to timer PWM output
        }

        if (DRV871x)                                                         // If device is DRV871xS/H-Q1
        {
            if (gBrushed_Obj.in3En == 100 && TBCCR5 == 0)
            {
                P4OUT &= ~IN3_EN;                                            // set the output to 0%
                P4SEL &= ~IN3_EN;                                            // Set IN3 to GPIO output
            }
            else if (gBrushed_Obj.in3En == 0 && TBCCR5 == 800)
            {
                P4OUT |= IN3_EN;                                             // set the output to 100%
                P4SEL &= ~IN3_EN;                                            // Set IN3 to GPIO output
            }
            else
            {
                drv_RampCTRL(&gBrushed_Obj.rampInput3, gBrushed_Obj.in3En);  //Ramp controller for en_in3
                TBCCR5 = 800 - gBrushed_Obj.rampInput3.setpointValue * 8;
                P4SEL |= IN3_EN;                                             // Set IN3 to timer PWM output
            }

            if (gBrushed_Obj.in4Ph == 100 && TBCCR6 == 0)
            {
                P4OUT &= ~IN4_PH;                                            // set the output to 0%
                P4SEL &= ~IN4_PH;                                            // Set IN4 to GPIO output
            }
            else if (gBrushed_Obj.in4Ph == 0 && TBCCR6 == 800)
            {
                P4OUT |= IN4_PH;                                             // set the output to 100%
                P4SEL &= ~IN4_PH;                                            // Set IN4 to GPIO output
            }
            else
            {
                drv_RampCTRL(&gBrushed_Obj.rampInput4, gBrushed_Obj.in4Ph);  //Ramp controller for ph_in4
                TBCCR6 = 800 - gBrushed_Obj.rampInput4.setpointValue * 8;
                P4SEL |= IN4_PH;                                             // Set IN4 to timer PWM output
            }
        }
    }
}

