/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  brushed.h - Declaration file for functions and global variables related to brushed control
 *
 ******************************************************************************/
#ifndef BRUSHED_H_
#define BRUSHED_H_
/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include <dataTypeDefinition.h>
#include <msp430.h>
/*****************************************************************************/
// Declare Global Variables
//***********************************************************************************
// Global Variables for GUI Composer                                                         *
//***********************************************************************************
/*
 * Ramp duty controller
 */
typedef struct DRV8x06_RMPCNTL
{
    volatile uint16_t rampDelayMax;            // Parameter: Maximum delay rate (0-255)
    volatile uint16_t rampDelayCount;          // Variable: Incremental delay (0-255)
    volatile uint16_t setpointValue;           // Output: Target output(0-100)

} RMPCNTL_t;

typedef struct Brushed_Driving_Obj
{
    /*Ramp controller*/
    RMPCNTL_t rampInput1;
    RMPCNTL_t rampInput2;

    volatile uint8_t in1En;
    volatile uint8_t nhiz1;
    volatile uint8_t in2Ph;
    volatile uint8_t nhiz2;

    volatile uint16_t ph;

    /* Control Feedback */
    volatile uint16_t ADC12read;

    volatile uint8_t startMotor;               // for starting or ending PWMs
    volatile uint8_t pwmState;
    volatile uint8_t pwmMode;

    /**< 0: HB1 motor connected to GND, 1: HB1 motor connected to PVDD,
     2: HB(s) configured in different way */
    volatile uint8_t hBMotorConn;

    volatile uint8_t soReadComplete;

    /*Ramp controller*/
    RMPCNTL_t rampInput3;
    RMPCNTL_t rampInput4;

    volatile uint8_t in3En;
    volatile uint8_t in4Ph;

    volatile uint16_t ph2;

    /* Control Feedback */
    volatile uint16_t ADC12read2;
    volatile uint8_t soReadComplete2;

} brushed_Driving_Obj_t;

/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
extern volatile brushed_Driving_Obj_t gBrushed_Obj;

/*****************************************************************************/

// Function Declarations

// GPIO/GUI Functions
void brushed_Init(void);

// Main Functions
void updateDeviceControls(void);
void updateSOReading(void);
void updateSOSenseAREF(void);
void updateSVariants(void);
void setDefaultGUIVals(void);
void drv_setCtrlMode(void);
void setDefault_GUI_DRV_Vals(void);
void updatePWM(void);

uint8_t getPWMMode(void);

void SetVariantFlags(void);
uint8_t SVariant;
uint8_t DRV871x;
uint8_t DRV8718S;                              // if 1 DRV8718S, if 0 DRV8714S or H
uint8_t nSLEEP_is_hi;
uint8_t register_updated;

#endif /*brushed_H_*/
