/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv8x06.h - Declaration file for utility functions and global variables
 *  DRV87xx_8106-Q1EVM_FIRMWARE
 *  10/25/2021
 *  Murugavel Raju
 *
 ******************************************************************************/
#ifndef drv8x06q1_H_
#define drv8x06q1_H_

/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "../../brushed.h"
#include "drv8x06-RegMap.h"
#include "msp430.h"

/******************************************************************************
 * MACROS
 *****************************************************************************/
//#define DEBUG                                // used for ADC12 debug only
//#define FIRMWARE_VERSION 1.4                 // Supported with GUI v0.5.3
#define FIRMWARE_VERSION 1.5                   // Supported with GUI v0.5.4 10/25/2021

#define PWM_FREQ_20KHZ            0
#define PWM_FREQ_10KHZ            ID0
#define PWM_FREQ_5KHZ             ID1
#define PWM_FREQ_2P5KHZ           ID1 | ID0

/*
 * GPIO Port 1 Definitions
 */
#define nFAULT_INT                BIT1         // P1.1 Fault indication pin (with port interrupt)
#define DRVOFF_nFLT_INT           BIT1         // P1.1 drv871x-q1
#define DRVOFF                    BIT2         // P1.2 DRVOFF enable pin
#define LED_DIS                   BIT2         // P1.2 drv871x-q1 pull low when LED disabled. pull high to enable LED
#define nSLEEP_pin                BIT3         // P1.3 Sleep pin
#define nHIZ2                     BIT4         // P1.4 nHIZ2 pin
#define BRAKE                     BIT4         // P1.4 drv871x-q1 MCU BRAKE function
#define nHIZ1                     BIT6         // P1.6 nHIZ1 pin

/*
 * GPIO Port 3 Definitions
 */
#define nSCS                      BIT0         // P3.0
#define SDI                       BIT1         // P3.1
#define SDO                       BIT2         // P3.2
#define SCLK                      BIT3         // P3.3
#define RX                        BIT4         // P3.4
#define TX                        BIT5         // P3.5
#define PUSH                      BIT6         // P3.6 PUSH Button pin
#define STATUS                    BIT7         // P3.7 STATUS LED pin

/*
 * GPIO Port 4 Definitions
 */
#define ADC_TIMER                 BIT1         // P4.0 Timer B Period
#define IN1_EN                    BIT2         // P4.1 IN1_EN pin
#define IN2_PH                    BIT3         // P4.2 IN2/PH pin
#define IN3_EN                    BIT5         // P4.5 drv871x-q1 IN3_EN pin
#define IN4_PH                    BIT6         // P4.6 drv871x-q1 IN4/PH pin

/*
 * GPIO Port 5 Definitions
 */
#define ID_DRV3                   BIT5         // P5.5 ID3 pin
#define ID_DRV2                   BIT6         // P5.6 ID2 pin
#define ID_DRV1                   BIT7         // P5.7 ID1 pin

/*
 * GPIO Port 6 Definitions
 */
#define POT                       BIT1         // P6.1 POT pin
#define SO2                       BIT1         // P6.1 drv871x-q1 MSP_SO2 pin (CSA 2)
#define MSP_SO2                   BIT2         // P6.2 MSP_SO2 pin (CSA 1)
#define SO1                       BIT2         // P6.1 drv871x-q1 MSP_SO1 pin (CSA 1)
#define POT2                      BIT3         // P6.3 drv871x-q1 POT pin

/* set LED_low as HIZ */
#define set_LED_DIS_HIZ           P1DIR &= ~LED_DIS

/* set LED_DIS low */
#define set_LED_DIS_lo            P1OUT &= ~LED_DIS; P1DIR |= LED_DIS

/* Enable nSLEEP*/
#define set_nSLEEP_hi             P1OUT |= nSLEEP_pin
#define set_nSLEEP_lo             P1OUT &= ~nSLEEP_pin

/* Enable DRVOFF*/
#define set_DRVOFF_hi             P1OUT |= DRVOFF
#define set_DRVOFF_lo             P1OUT &= ~DRVOFF

/* Enable BRAKE*/
#define set_BRAKE_hi              P1OUT |= BRAKE
#define set_BRAKE_lo              P1OUT &= ~BRAKE

/* Enable nHIZ2*/
#define set_nHIZ2_hi              P1OUT |= nHIZ2
#define set_nHIZ2_lo              P1OUT &= ~nHIZ2

/* Enable IN2_PH*/
#define set_IN2_PH_hi             P4OUT |= IN2_PH
#define set_IN2_PH_lo             P4OUT &= ~IN2_PH

/* Enable nHIZ1*/
#define set_nHIZ1_hi              P1OUT |= nHIZ1
#define set_nHIZ1_lo              P1OUT &= ~nHIZ1

/* Enable IN1_EN*/
#define set_IN1_EN_hi             P4OUT |= IN1_EN
#define set_IN1_EN_lo             P4OUT &= ~IN1_EN

#define read_nFAULT_INT_pin       P1IN & nFAULT_INT
#define read_ID_DRV1_pin          P5IN & ID_DRV1
#define read_ID_DRV2_pin          P5IN & ID_DRV2
#define read_ID_DRV3_pin          P5IN & ID_DRV3
#define toggle_status_led         P3OUT ^=  STATUS
#define toggle_push               P3OUT ^=  PUSH

#define set_PUSH_hi               P3OUT |= PUSH
#define set_PUSH_lo               P3OUT &= ~PUSH

#define set_DRVOFF_nFLT_INT_hi    P1OUT |= DRVOFF_nFLT_INT
#define set_DRVOFF_nFLT_INT_lo    P1OUT &= ~DRVOFF_nFLT_INT
#define read_DRVOFF_nFLT_INT_pin  P1IN & DRVOFF_nFLT_INT

/* Enable IN4_PH*/
#define set_IN4_PH_hi             P4OUT |= IN4_PH
#define set_IN4_PH_lo             P4OUT &= ~IN4_PH

/* Enable IN3_EN*/
#define set_IN3_EN_hi             P4OUT |= IN3_EN
#define set_IN3_EN_lo             P4OUT &= ~IN3_EN

#define SOSense_DEFAULT 2048
#define SOSense2_DEFAULT 2048
#define SOSense_DIV8_DEFAULT 512
#define SOSense2_DIV8_DEFAULT 512
#define TBCCR1_DEFAULT 4
#define TBCCR1_HB_DEFAULT 800
#define NumSamples 128

/******************************************************************************
 * DATA STRUCTURES
 *****************************************************************************/

typedef struct DRV8x06_Device_Obj
{
    /* Firmware Parameters*/
    uint8_t deviceID;
    uint8_t landingPage;
    float firmwareVersion;

    /* Device Parameters*/
    volatile uint8_t nSleep;                   // 0: Device Sleep , 1: Device Awake
    volatile uint8_t drvOFF;                   // 0: Outputs active, 1: Outputs Hi-Z
    volatile uint8_t nFault;                   // 0: No fault , 1: Fault
    volatile uint8_t clrFlt;                   // not utilized in the firmware

    /* Register Read Write */
    uint16_t ManWriteAddr;
    uint16_t ManReadAddr;
    uint16_t ManWriteData;
    uint16_t ManReadData;
    bool ManWriteCmd;
    bool ManReadCmd;
    bool ReadCmd;
    bool WriteCmd;
    uint8_t revID;                             //ID IC revision
} DRV8x06_Device_Obj_t;

/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
extern volatile DRV8x06_Q1_REG_t gDeviceSpiReg;
extern volatile DRV8x06_Device_Obj_t gDRV8x06_Obj;

/*****************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************
 * API Prototypes
 *****************************************************************************/
void clock_Init(void);
void gpio_Init(void);
void drv_deviceID(void);
void gpio_Init_Drv871x(void);
void gpio_Init_Drv870x_8106(void);
void pwmTimer_Init(void);
void adc12_Init(void);
void watchdogTimer_Init(void);

void update_DRVRegs(void);
void update_Regs(void);
void reg_Init(void);

void drv_IHBRampControl(void);
void drv_PH_EN_RampControl(void);
void drv_PWM_RampDown(void);
void drv_RampCTRL(volatile RMPCNTL_t* rmpCntl, uint8_t targetValue);
void drv_MCU_GUI_Init(void);

/*****************************************************************************/

/* Support for C++ codebase */
#ifdef __cplusplus
}
#endif

#endif
