/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  DRV8x06.c - Definition file for utility functions
 *  DRV87xx_8106-Q1EVM_FIRMWARE
 *  10/25/2021
 *  Murugavel Raju
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include <brushed.h>
#include <drv8x06.h>
#include <SPI.h>
#include "uart.h"
/********************************************************************************
 * MACROS
 *******************************************************************************/
#pragma DATA_SECTION(gDRV8x06_Obj,".drvVarSection");
#pragma DATA_SECTION(gDeviceSpiReg,".drvRegSection");
volatile DRV8x06_Q1_REG_t gDeviceSpiReg;
volatile DRV8x06_Device_Obj_t gDRV8x06_Obj;

/********************************************************************************
 * DATA
 *******************************************************************************/

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
int ID_DRV1_pin_resDis = 0;
int ID_DRV1_pin_resEn = 0;
int ID_DRV2_pin_resDis = 0;
int ID_DRV2_pin_resEn = 0;
int ID_DRV3_pin_resDis = 0;
int ID_DRV3_pin_resEn = 0;
int flag;
int pre_ph = 0;
int pre_ph2 = 0;
extern int timerUpdate;

/*
 * Clock Initialization
 *
 */
void clock_Init(void)
{
    // Stop Watchdog Timer
    WDTCTL = WDTPW | WDTHOLD;
    DCOCTL = CALDCO_16MHZ;
    BCSCTL1 = CALBC1_16MHZ;
    // 16MHz SMCLK for SMCLK frequency
    BCSCTL2 |= DIVS_0;
}

/*
 * GPIO Initialization. This block configures all the MCU PINs.
 */
void gpio_Init(void)
{

    // Note: All unused ports should be set to either output or input with
    // pullup/down enabled

    P2OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All high
    P2DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All OUTPUT

    P3OUT = PUSH | TX | nSCS;
    P3REN = PUSH;
    P3DIR = STATUS | nSCS;

    P5OUT = ID_DRV1 | ID_DRV2 | ID_DRV3 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;  // All high
    P5DIR = BIT4 | BIT3 | BIT2 | BIT1 | BIT0;                                // All OUTPUT except all ID_DRVx signals
    P5REN |= ID_DRV1 | ID_DRV2 | ID_DRV3;

    P7OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All high
    P7DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All OUTPUT

    P8OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All high
    P8DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All OUTPUT

    PAOUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All high
    PADIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All OUTPUT

}

void drv_deviceID()
{
    /* Identify the Device connected */

    // drv8706s-q1 deviceID = 1
    // drv8706h-q1 deviceID = 2
    // drv8106s-q1 deviceID = 3
    // drv8106h-q1 deviceID = 4
    // drv8705s-q1 deviceID = 5
    // drv8705h-q1 deviceID = 6
    // drv8718s-q1 deviceID = 7
    // drv8714s-q1 deviceID = 8
    // drv8714h-q1 deviceID = 9

    // POUT is high
    ID_DRV1_pin_resDis = read_ID_DRV1_pin;
    ID_DRV2_pin_resDis = read_ID_DRV2_pin;
    ID_DRV3_pin_resDis = read_ID_DRV3_pin;

    P5OUT &= ~(ID_DRV1 | ID_DRV2 | ID_DRV3);
    __delay_cycles(8000);

    ID_DRV1_pin_resEn = read_ID_DRV1_pin;
    ID_DRV2_pin_resEn = read_ID_DRV2_pin;
    ID_DRV3_pin_resEn = read_ID_DRV3_pin;

    //drv8706s-q1
    if (ID_DRV1_pin_resDis == 128 && ID_DRV1_pin_resEn == 128
            && ID_DRV2_pin_resDis != ID_DRV2_pin_resEn
            && ID_DRV3_pin_resDis != ID_DRV3_pin_resEn)
        gDRV8x06_Obj.deviceID = 1;
    //drv8706h-q1
    else if (ID_DRV1_pin_resDis == 0 && ID_DRV1_pin_resEn == 0
            && ID_DRV2_pin_resDis != ID_DRV2_pin_resEn
            && ID_DRV3_pin_resDis != ID_DRV3_pin_resEn)
        gDRV8x06_Obj.deviceID = 2;
    //drv8106s-q1
    else if (ID_DRV1_pin_resDis != ID_DRV1_pin_resEn && ID_DRV2_pin_resDis == 0
            && ID_DRV2_pin_resEn == 0 && ID_DRV3_pin_resDis == 0
            && ID_DRV3_pin_resEn == 0)
        gDRV8x06_Obj.deviceID = 3;
    //drv8106h-q1
    else if (ID_DRV1_pin_resDis != ID_DRV1_pin_resEn && ID_DRV2_pin_resDis == 64
            && ID_DRV2_pin_resEn == 64
            && ID_DRV3_pin_resDis != ID_DRV3_pin_resEn)
        gDRV8x06_Obj.deviceID = 4;
    //drv8705s-q1
    else if (ID_DRV1_pin_resDis != ID_DRV1_pin_resEn
            && ID_DRV2_pin_resDis != ID_DRV2_pin_resEn
            && ID_DRV3_pin_resDis != ID_DRV3_pin_resEn)
        gDRV8x06_Obj.deviceID = 5;
    //drv8705h-q1
    else if (ID_DRV1_pin_resDis == 0 && ID_DRV1_pin_resEn == 0
            && ID_DRV2_pin_resDis != ID_DRV2_pin_resEn
            && ID_DRV3_pin_resDis == 0 && ID_DRV3_pin_resEn == 0)
        gDRV8x06_Obj.deviceID = 6;
    //drv8718s-q1
    else if (ID_DRV1_pin_resDis != ID_DRV1_pin_resEn && ID_DRV2_pin_resDis == 0
            && ID_DRV2_pin_resEn == 0
            && ID_DRV3_pin_resDis != ID_DRV3_pin_resEn)
        gDRV8x06_Obj.deviceID = 7;
    //drv8714s-q1
    else if (ID_DRV1_pin_resDis != ID_DRV1_pin_resEn
            && ID_DRV2_pin_resDis != ID_DRV2_pin_resEn
            && ID_DRV3_pin_resDis == 32 && ID_DRV3_pin_resEn == 32)
        gDRV8x06_Obj.deviceID = 8;
    //drv8714h-q1
    else if (ID_DRV1_pin_resDis != ID_DRV1_pin_resEn
            && ID_DRV2_pin_resDis != ID_DRV2_pin_resEn
            && ID_DRV3_pin_resDis == 0 && ID_DRV3_pin_resEn == 0)
        gDRV8x06_Obj.deviceID = 9;
} // End drv_deviceID()

/*
 * GPIO Initialization. This block configures all the MCU PINs.
 */
void gpio_Init_Drv870x_8106(void)
{
    P1OUT = BIT7 | BIT5 | nFAULT_INT | BIT0;
    P1DIR = BIT7 | nHIZ1 | BIT5 | nHIZ2 | nSLEEP_pin | DRVOFF | BIT0;

    P1IE |= nFAULT_INT;                                                      // P1.4 interrupt enabled
    P1IES |= nFAULT_INT;                                                     // P1.4 Hi/Lo edge
    P1IFG &= ~nFAULT_INT;                                                    // P1.4 IFG cleared

    P4OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT0;                                // All high except timer pins
    P4DIR = BIT7 | BIT6 | BIT5 | BIT4 | IN2_PH | IN1_EN | ADC_TIMER | BIT0;  // All OUTPUT
    P4SEL |= IN2_PH | IN1_EN | ADC_TIMER;                                    // P4.1 - P4.3 TBx options

    P6OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | MSP_SO2 | POT | BIT0;
    P6DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | POT | BIT0;
    P6SEL = MSP_SO2;             // select A2 input as ADC1 function
}

/*
 * GPIO Initialization. This block configures all the MCU PINs.
 */
void gpio_Init_Drv871x(void)
{
    if (gDRV8x06_Obj.deviceID == 9)                                          // H variant has only nfault reporting, no drvoff
    {
        P1OUT = BIT7 | BIT6 | BIT5 | DRVOFF_nFLT_INT | BIT0;
        P1DIR = BIT7 | BIT6 | BIT5 | BRAKE | nSLEEP_pin | BIT0;              // Set LED disable to high z for H variant
        P1IE |= DRVOFF_nFLT_INT;                                             // P1.4 interrupt enabled
        P1IES |= DRVOFF_nFLT_INT;                                            // P1.4 Hi/Lo edge
        P1IFG &= ~DRVOFF_nFLT_INT;                                           // P1.4 IFG cleared

        P6DIR = POT2 | BIT0;
    }
    else
    {
        P1OUT = BIT7 | BIT6 | BIT5 | BIT0;                                   // LED_DIS low to disable LED in S variant
        P1DIR = BIT7 | BIT6 | BIT5 | BRAKE | nSLEEP_pin | LED_DIS | DRVOFF_nFLT_INT | BIT0; // Setup as DRVOFF output pin initially

        P6DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT0;
    }

    P4OUT = BIT7 | BIT4 | BIT0;                                              // All high except timer pins
    P4DIR = BIT7 | IN4_PH | IN3_EN | BIT4 | IN2_PH | IN1_EN | ADC_TIMER | BIT0; // All OUTPUT
    P4SEL |= IN4_PH | IN3_EN | IN2_PH | IN1_EN | ADC_TIMER;                  // P4.1 - P4.3 TBx options

    P6OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | SO1 | SO2 | BIT0;
    P6DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT0;
    P6SEL = POT | SO1 | SO2;                                                 // select A2 input as ADC1 function, A1 input as ADC2 function

}

/*
 * PWM control timer Initialization. Atomic setup. Provide PWM to control input EN_IN1 & PH_EN2 (Period 50us).
 */
void pwmTimer_Init(void)
{
    // Initialize Timer B0 with 0% duty for IN1 and EN2
    TBCTL = TBSSEL_2 | MC_0 | TBCLR;                                         // Timer clock is SMCLK 16MHz, timer halted and cleared
    TBCCTL0 |= CCIE;                                                         // Interrupt enable
    TBCCR0 = 800;                                                            // Timer period value

    TBCCTL1 |= OUTMOD_7;                                                     // Timer mode to PWM reset/set for ADC12 trigger
    TBCCR1 = TBCCR1_DEFAULT;                                                 // ADC12 trigger compare register default value

    TBCCTL2 |= OUTMOD_7;                                                     // Timer mode to PWM reset/set for IN1
    TBCCTL3 |= OUTMOD_7;                                                     // Timer mode to PWM reset/set for IN2
    TBCCR2 = 0;                                                              // IN1 compare register reset
    TBCCR3 = 0;                                                              // IN2 compare register reset

    if (DRV871x)                                                             // Device is DRV871x-Q1
    {
        TBCCTL5 |= OUTMOD_7;                                                 // Timer mode to PWM reset/set for IN3
        TBCCTL6 |= OUTMOD_7;                                                 // Timer mode to PWM reset/set for IN4
        TBCCR5 = 0;                                                          // IN3 compare register reset
        TBCCR6 = 0;                                                          // IN4 compare register reset
    }

    TBCTL = TBSSEL_2 | PWM_FREQ_20KHZ | MC_1 | TBCLR;                        // Timer clock is SMCLK 16MHz, up mode, cleared to start counting from zero
}

/*
 * ADC12 Initialization. Capture the SO voltage value in the ADC12 ISR.
 */
void adc12_Init(void)
{
    ADC12CTL0 = ADC12ON;                                                     // Setup ADC12
    ADC12CTL1 = CONSEQ_3 + SHS_3;                                            // Timer triggers sampling

#ifdef DEBUG
    ADC12MCTL0 = INCH_3 + SREF_2;                                            // Used for potentiometer for debug
    ADC12MCTL1 = EOS;                                                        // End of sequence
#endif

    if (DRV871x == 0)
    {
        ADC12MCTL0 = EOS + INCH_2 + SREF_2;                                  // A2 for SO1 with VeRef+ reference voltage. End of sequence.
        ADC12IE = BIT0;                                                      // Enable ADC Interrupt after sequence complete
    }

    if (DRV871x)                                                             // Device is DRV871x-Q1
    {
        ADC12MCTL0 = INCH_2 + SREF_2;                                        // A2 for SO1 with VeRef+ reference voltage
        ADC12MCTL1 = EOS + INCH_1 + SREF_2;                                  // A1 for SO2 with VeRef+ reference voltage. End of sequence.
        ADC12IE = BIT1;                                                      // Enable ADC Interrupt after sequence complete
    }

    ADC12CTL0 |= ENC;                                                        // Conversion enabled
}
/*
 * Setup Watchdog Timer as an Interval Timer(250ms)
 * Timer Interval to Blink the STATUS LED
 */
void watchdogTimer_Init(void)
{
    WDTCTL = WDT_MDLY_32;                                                    // Configure time interval
    IE1 |= WDTIE;                                                            // Interrupt enable
}

void update_DRVRegs()
{
    register_updated = 1;
    if (DRV871x)                                                             // Device is DRV8714S-Q1 or DRV8718S-Q1
    {

        SPI_writeRegister(IC_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.ic_ctrl1_reg);

        SPI_writeRegister(IC_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.ic_ctrl2_reg);

        SPI_writeRegister(BRG_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.brg_ctrl1_reg);
        SPI_writeRegister(BRG_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.brg_ctrl2_reg);

        SPI_writeRegister(PWM_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pwm_ctrl1_reg);
        SPI_writeRegister(PWM_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pwm_ctrl2_reg);
        SPI_writeRegister(PWM_CTRL3_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pwm_ctrl3_reg);
        SPI_writeRegister(PWM_CTRL4_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pwm_ctrl4_reg);

        SPI_writeRegister(IDRV_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl1_reg);
        SPI_writeRegister(IDRV_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl2_reg);
        SPI_writeRegister(IDRV_CTRL3_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl3_reg);
        SPI_writeRegister(IDRV_CTRL4_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl4_reg);
        SPI_writeRegister(IDRV_CTRL5_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl5_reg);
        SPI_writeRegister(IDRV_CTRL6_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl6_reg);
        SPI_writeRegister(IDRV_CTRL7_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl7_reg);
        SPI_writeRegister(IDRV_CTRL8_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl8_reg);
        SPI_writeRegister(IDRV_CTRL9_DRV871XX,
                          (uint16_t) gDeviceSpiReg.idrv_ctrl9_reg);

        SPI_writeRegister(DRV_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.drv_ctrl1_reg);
        SPI_writeRegister(DRV_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.drv_ctrl2_reg);
        SPI_writeRegister(DRV_CTRL3_DRV871XX,
                          (uint16_t) gDeviceSpiReg.drv_ctrl3_reg);
        SPI_writeRegister(DRV_CTRL4_DRV871XX,
                          (uint16_t) gDeviceSpiReg.drv_ctrl4_reg);
        SPI_writeRegister(DRV_CTRL5_DRV871XX,
                          (uint16_t) gDeviceSpiReg.drv_ctrl5_reg);
        SPI_writeRegister(DRV_CTRL6_DRV871XX,
                          (uint16_t) gDeviceSpiReg.drv_ctrl6_reg);
        SPI_writeRegister(DRV_CTRL7_DRV871XX,
                          (uint16_t) gDeviceSpiReg.drv_ctrl7_reg);

        SPI_writeRegister(VDS_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.vds_ctrl1_reg);
        SPI_writeRegister(VDS_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.vds_ctrl2_reg);
        SPI_writeRegister(VDS_CTRL3_DRV871XX,
                          (uint16_t) gDeviceSpiReg.vds_ctrl3_reg);
        SPI_writeRegister(VDS_CTRL4_DRV871XX,
                          (uint16_t) gDeviceSpiReg.vds_ctrl4_reg);

        SPI_writeRegister(OLSC_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.olsc_ctrl1_reg);
        SPI_writeRegister(OLSC_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.olsc_ctrl2_reg);

        SPI_writeRegister(UVOV_CTRL_DRV871XX,
                          (uint16_t) gDeviceSpiReg.uvov_ctrl_reg);

        SPI_writeRegister(CSA_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.csa_ctrl1_reg);
        SPI_writeRegister(CSA_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.csa_ctrl2_reg);
        SPI_writeRegister(CSA_CTRL3_DRV871XX,
                          (uint16_t) gDeviceSpiReg.csa_ctrl3_reg);

        SPI_writeRegister(AGD_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.agd_ctrl1_reg);

        SPI_writeRegister(PDR_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl1_reg);
        SPI_writeRegister(PDR_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl2_reg);
        SPI_writeRegister(PDR_CTRL3_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl3_reg);
        SPI_writeRegister(PDR_CTRL4_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl4_reg);
        SPI_writeRegister(PDR_CTRL5_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl5_reg);
        SPI_writeRegister(PDR_CTRL6_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl6_reg);
        SPI_writeRegister(PDR_CTRL7_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl7_reg);
        SPI_writeRegister(PDR_CTRL8_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl8_reg);
        SPI_writeRegister(PDR_CTRL9_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl9_reg);
        SPI_writeRegister(PDR_CTRL10_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pdr_ctrl10_reg);

        SPI_writeRegister(STC_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.stc_ctrl1_reg);
        SPI_writeRegister(STC_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.stc_ctrl2_reg);
        SPI_writeRegister(STC_CTRL3_DRV871XX,
                          (uint16_t) gDeviceSpiReg.stc_ctrl3_reg);
        SPI_writeRegister(STC_CTRL4_DRV871XX,
                          (uint16_t) gDeviceSpiReg.stc_ctrl4_reg);

        SPI_writeRegister(DCC_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.dcc_ctrl1_reg);

        SPI_writeRegister(PST_CTRL1_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pst_ctrl1_reg);
        SPI_writeRegister(PST_CTRL2_DRV871XX,
                          (uint16_t) gDeviceSpiReg.pst_ctrl2_reg);
    }
    else                                                                     // Device is DRV8106S-Q1 or DRV8705S-Q1 or DRV8706S -Q1
    {
        SPI_writeRegister(IC_CTRL, (uint16_t) gDeviceSpiReg.ic_ctrl_reg);
        SPI_writeRegister(BRG_CTRL, (uint16_t) gDeviceSpiReg.brg_ctrl_reg);
        SPI_writeRegister(DRV_CTRL_1, (uint16_t) gDeviceSpiReg.drv_ctrl_1_reg);
        SPI_writeRegister(DRV_CTRL_2, (uint16_t) gDeviceSpiReg.drv_ctrl_2_reg);
        SPI_writeRegister(DRV_CTRL_3, (uint16_t) gDeviceSpiReg.drv_ctrl_3_reg);
        SPI_writeRegister(VDS_CTRL_1, (uint16_t) gDeviceSpiReg.vds_ctrl_1_reg);
        SPI_writeRegister(VDS_CTRL_2, (uint16_t) gDeviceSpiReg.vds_ctrl_2_reg);
        SPI_writeRegister(OLSC_CTRL, (uint16_t) gDeviceSpiReg.olsc_ctrl_reg);
        SPI_writeRegister(UVOV_CTRL, (uint16_t) gDeviceSpiReg.uvov_ctrl_reg);
        SPI_writeRegister(CSA_CTRL, (uint16_t) gDeviceSpiReg.csa_ctrl_reg);
    }

}

void update_Regs()                                                           // Routine for updating registers from the GUI
{
    uint16_t drvDataNew;
    uint8_t drvRegAddr;

    if (gDRV8x06_Obj.nSleep)                                                 // If device is awake
    {
        /* Clear Fault is initiated in the GUI */
        if (DRV871x)                                                         // If device is DRV8714S-Q1 or DRV8718S-Q1
        {
            if (gDeviceSpiReg.ic_ctrl1_reg & CLR_FLT_MASK)
            {
                SPI_writeRegister(IC_CTRL1_DRV871XX,
                                  (uint16_t) gDeviceSpiReg.ic_ctrl1_reg);

                gDeviceSpiReg.ic_ctrl1_reg = SPI_readRegister(
                        (uint8_t) IC_CTRL1_DRV871XX);
            }
        }
        else                                                                 // Else if device is DRV8106S-Q1 or DRV8705S-Q1 or DRV8706S -Q1
        {
            if (gDeviceSpiReg.ic_ctrl_reg & CLR_FLT_MASK)
            {
                SPI_writeRegister(IC_CTRL, (uint16_t) gDeviceSpiReg.ic_ctrl_reg);
                gDeviceSpiReg.ic_ctrl_reg = SPI_readRegister((uint8_t) IC_CTRL);
            }
        }
        /* End Clear Fault*/

        if (gDRV8x06_Obj.WriteCmd)
        {
            register_updated = 1;
            update_DRVRegs();
            gDRV8x06_Obj.WriteCmd = false;
        }

        if (gDRV8x06_Obj.ManWriteCmd)
        {
            register_updated = 1;
            // Custom Write
            drvRegAddr = gDRV8x06_Obj.ManWriteAddr;
            drvDataNew = gDRV8x06_Obj.ManWriteData;

            SPI_writeRegister((uint8_t) drvRegAddr, (uint16_t) drvDataNew);

            if (DRV871x)                                                     // Device is DRV8714S-Q1 or DRV8718S-Q1
            {
                switch (drvRegAddr)
                {

                case IC_STAT_1_DRV871XX :
                    gDeviceSpiReg.ic_stat1_reg = drvDataNew;
                    break;

                case VDS_STAT1_DRV871XX :
                    gDeviceSpiReg.vds_stat1_reg = drvDataNew;
                    break;

                case VDS_STAT2_DRV871XX :
                    gDeviceSpiReg.vds_stat2_reg = drvDataNew;
                    break;

                case VGS_STAT1_DRV871XX :
                    gDeviceSpiReg.vgs_stat1_reg = drvDataNew;
                    break;

                case VGS_STAT2_DRV871XX :
                    gDeviceSpiReg.vgs_stat2_reg = drvDataNew;
                    break;

                case IC_STAT2_DRV871XX :
                    gDeviceSpiReg.ic_stat2_reg = drvDataNew;
                    break;

                case IC_STAT3_DRV871XX :
                    gDeviceSpiReg.ic_stat3_reg = drvDataNew;
                    break; // Read only Registers

                case IC_CTRL1_DRV871XX :
                    gDeviceSpiReg.ic_ctrl1_reg = drvDataNew;
                    break;

                case IC_CTRL2_DRV871XX :
                    gDeviceSpiReg.ic_ctrl2_reg = drvDataNew;
                    break;

                case BRG_CTRL1_DRV871XX :
                    gDeviceSpiReg.brg_ctrl1_reg = drvDataNew;
                    break;

                case BRG_CTRL2_DRV871XX :
                    gDeviceSpiReg.brg_ctrl2_reg = drvDataNew;
                    break;

                case PWM_CTRL1_DRV871XX :
                    gDeviceSpiReg.pwm_ctrl1_reg = drvDataNew;
                    break;

                case PWM_CTRL2_DRV871XX :
                    gDeviceSpiReg.pwm_ctrl2_reg = drvDataNew;
                    break;

                case PWM_CTRL3_DRV871XX :
                    gDeviceSpiReg.pwm_ctrl3_reg = drvDataNew;
                    break;

                case PWM_CTRL4_DRV871XX :
                    gDeviceSpiReg.pwm_ctrl4_reg = drvDataNew;
                    break;

                case IDRV_CTRL1_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl1_reg = drvDataNew;
                    break;

                case IDRV_CTRL2_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl2_reg = drvDataNew;
                    break;

                case IDRV_CTRL3_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl3_reg = drvDataNew;
                    break;

                case IDRV_CTRL4_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl4_reg = drvDataNew;
                    break;

                case IDRV_CTRL5_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl5_reg = drvDataNew;
                    break;

                case IDRV_CTRL6_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl6_reg = drvDataNew;
                    break;

                case IDRV_CTRL7_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl7_reg = drvDataNew;
                    break;

                case IDRV_CTRL8_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl8_reg = drvDataNew;
                    break;

                case IDRV_CTRL9_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl9_reg = drvDataNew;
                    break;

                case DRV_CTRL1_DRV871XX :
                    gDeviceSpiReg.drv_ctrl1_reg = drvDataNew;
                    break;

                case DRV_CTRL2_DRV871XX :
                    gDeviceSpiReg.drv_ctrl2_reg = drvDataNew;
                    break;

                case DRV_CTRL3_DRV871XX :
                    gDeviceSpiReg.drv_ctrl3_reg = drvDataNew;
                    break;

                case DRV_CTRL4_DRV871XX :
                    gDeviceSpiReg.drv_ctrl4_reg = drvDataNew;
                    break;

                case DRV_CTRL5_DRV871XX :
                    gDeviceSpiReg.drv_ctrl5_reg = drvDataNew;
                    break;

                case DRV_CTRL6_DRV871XX :
                    gDeviceSpiReg.drv_ctrl6_reg = drvDataNew;
                    break;

                case DRV_CTRL7_DRV871XX :
                    gDeviceSpiReg.drv_ctrl7_reg = drvDataNew;
                    break;

                case VDS_CTRL1_DRV871XX :
                    gDeviceSpiReg.vds_ctrl1_reg = drvDataNew;
                    break;

                case VDS_CTRL2_DRV871XX :
                    gDeviceSpiReg.vds_ctrl2_reg = drvDataNew;
                    break;

                case VDS_CTRL3_DRV871XX :
                    gDeviceSpiReg.vds_ctrl3_reg = drvDataNew;
                    break;

                case VDS_CTRL4_DRV871XX :
                    gDeviceSpiReg.vds_ctrl4_reg = drvDataNew;
                    break;

                case OLSC_CTRL1_DRV871XX :
                    gDeviceSpiReg.olsc_ctrl1_reg = drvDataNew;
                    break;

                case OLSC_CTRL2_DRV871XX :
                    gDeviceSpiReg.olsc_ctrl2_reg = drvDataNew;
                    break;

                case UVOV_CTRL_DRV871XX :
                    gDeviceSpiReg.uvov_ctrl_reg = drvDataNew;
                    break;

                case CSA_CTRL1_DRV871XX :
                    gDeviceSpiReg.csa_ctrl1_reg = drvDataNew;
                    break;

                case CSA_CTRL2_DRV871XX :
                    gDeviceSpiReg.csa_ctrl2_reg = drvDataNew;
                    break;

                case CSA_CTRL3_DRV871XX :
                    gDeviceSpiReg.csa_ctrl3_reg = drvDataNew;
                    break;

                case AGD_CTRL1_DRV871XX :
                    gDeviceSpiReg.agd_ctrl1_reg = drvDataNew;
                    break;

                case PDR_CTRL1_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl1_reg = drvDataNew;
                    break;

                case PDR_CTRL2_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl2_reg = drvDataNew;
                    break;

                case PDR_CTRL3_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl3_reg = drvDataNew;
                    break;

                case PDR_CTRL4_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl4_reg = drvDataNew;
                    break;

                case PDR_CTRL5_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl5_reg = drvDataNew;
                    break;

                case PDR_CTRL6_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl6_reg = drvDataNew;
                    break;

                case PDR_CTRL7_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl7_reg = drvDataNew;
                    break;

                case PDR_CTRL8_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl8_reg = drvDataNew;
                    break;

                case PDR_CTRL9_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl9_reg = drvDataNew;
                    break;

                case PDR_CTRL10_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl10_reg = drvDataNew;
                    break;

                case STC_CTRL1_DRV871XX :
                    gDeviceSpiReg.stc_ctrl1_reg = drvDataNew;
                    break;

                case STC_CTRL2_DRV871XX :
                    gDeviceSpiReg.stc_ctrl2_reg = drvDataNew;
                    break;

                case STC_CTRL3_DRV871XX :
                    gDeviceSpiReg.stc_ctrl3_reg = drvDataNew;
                    break;

                case STC_CTRL4_DRV871XX :
                    gDeviceSpiReg.stc_ctrl4_reg = drvDataNew;
                    break;

                case DCC_CTRL1_DRV871XX :
                    gDeviceSpiReg.dcc_ctrl1_reg = drvDataNew;
                    break;

                case PST_CTRL1_DRV871XX :
                    gDeviceSpiReg.pst_ctrl1_reg = drvDataNew;
                    break;

                case PST_CTRL2_DRV871XX :
                    gDeviceSpiReg.pst_ctrl2_reg = drvDataNew;
                    break;

                case SGD_STAT1_DRV871XX :
                    gDeviceSpiReg.sgd_stat1_reg = drvDataNew;
                    break;

                case SGD_STAT2_DRV871XX :
                    gDeviceSpiReg.sgd_stat2_reg = drvDataNew;
                    break;

                case SGD_STAT3_DRV871XX :
                    gDeviceSpiReg.sgd_stat3_reg = drvDataNew;
                    break;

                default:
                    break;

                } // End of Switch
            }
            else                                                             // Device is DRV8106S-Q1 or DRV8705S-Q1 or DRV8706S -Q1
            {
                switch (drvRegAddr)
                {

                case IC_STAT_1 :
                    gDeviceSpiReg.ic_status_1_reg = drvDataNew;
                    break;

                case VGS_VDS_STAT :
                    gDeviceSpiReg.vgs_vds_stat_reg = drvDataNew;
                    break;

                case IC_STAT_2 :
                    gDeviceSpiReg.ic_status_2_reg = drvDataNew;
                    break; // Read only Registers

                case IC_CTRL :
                    gDeviceSpiReg.ic_ctrl_reg = drvDataNew;
                    break;

                case BRG_CTRL :
                    gDeviceSpiReg.brg_ctrl_reg = drvDataNew;
                    break;

                case DRV_CTRL_1 :
                    gDeviceSpiReg.drv_ctrl_1_reg = drvDataNew;
                    break;

                case DRV_CTRL_2 :
                    gDeviceSpiReg.drv_ctrl_2_reg = drvDataNew;
                    break;

                case DRV_CTRL_3 :
                    gDeviceSpiReg.drv_ctrl_3_reg = drvDataNew;
                    break;

                case VDS_CTRL_1 :
                    gDeviceSpiReg.vds_ctrl_1_reg = drvDataNew;
                    break;

                case VDS_CTRL_2 :
                    gDeviceSpiReg.vds_ctrl_2_reg = drvDataNew;
                    break;

                case OLSC_CTRL :
                    gDeviceSpiReg.olsc_ctrl_reg = drvDataNew;
                    break;

                case UVOV_CTRL :
                    gDeviceSpiReg.uvov_ctrl_reg = drvDataNew;
                    break;

                case CSA_CTRL :
                    gDeviceSpiReg.csa_ctrl_reg = drvDataNew;
                    break;

                default:
                    break;

                } // End of Switch
            }
            gDRV8x06_Obj.ManWriteCmd = false;
        } // End of Write Cmd

        if (gDRV8x06_Obj.ReadCmd)
        {
            if (DRV871x)                                                     // Device is DRV8714S-Q1 or DRV8718S-Q1
            {
                gDeviceSpiReg.ic_stat1_reg = SPI_readRegister(
                        (uint8_t) IC_STAT_1_DRV871XX);

                gDeviceSpiReg.vds_stat1_reg = SPI_readRegister(
                        (uint8_t) VDS_STAT1_DRV871XX);

                gDeviceSpiReg.vds_stat2_reg = SPI_readRegister(
                        (uint8_t) VDS_STAT2_DRV871XX);

                gDeviceSpiReg.vgs_stat1_reg = SPI_readRegister(
                        (uint8_t) VGS_STAT1_DRV871XX);

                gDeviceSpiReg.vgs_stat2_reg = SPI_readRegister(
                        (uint8_t) VGS_STAT2_DRV871XX);

                gDeviceSpiReg.ic_stat2_reg = SPI_readRegister(
                        (uint8_t) IC_STAT2_DRV871XX);

                gDeviceSpiReg.ic_stat3_reg = SPI_readRegister(
                        (uint8_t) IC_STAT3_DRV871XX);

                gDeviceSpiReg.ic_ctrl1_reg = SPI_readRegister(
                        (uint8_t) IC_CTRL1_DRV871XX);

                gDeviceSpiReg.ic_ctrl2_reg = SPI_readRegister(
                        (uint8_t) IC_CTRL2_DRV871XX);

                gDeviceSpiReg.brg_ctrl1_reg = SPI_readRegister(
                        (uint8_t) BRG_CTRL1_DRV871XX);

                gDeviceSpiReg.brg_ctrl2_reg = SPI_readRegister(
                        (uint8_t) BRG_CTRL2_DRV871XX);

                gDeviceSpiReg.pwm_ctrl1_reg = SPI_readRegister(
                        (uint8_t) PWM_CTRL1_DRV871XX);

                gDeviceSpiReg.pwm_ctrl2_reg = SPI_readRegister(
                        (uint8_t) PWM_CTRL2_DRV871XX);

                gDeviceSpiReg.pwm_ctrl3_reg = SPI_readRegister(
                        (uint8_t) PWM_CTRL3_DRV871XX);

                gDeviceSpiReg.pwm_ctrl4_reg = SPI_readRegister(
                        (uint8_t) PWM_CTRL4_DRV871XX);

                gDeviceSpiReg.idrv_ctrl1_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL1_DRV871XX);

                gDeviceSpiReg.idrv_ctrl2_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL2_DRV871XX);

                gDeviceSpiReg.idrv_ctrl3_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL3_DRV871XX);

                gDeviceSpiReg.idrv_ctrl4_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL4_DRV871XX);

                gDeviceSpiReg.idrv_ctrl5_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL5_DRV871XX);

                gDeviceSpiReg.idrv_ctrl6_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL6_DRV871XX);

                gDeviceSpiReg.idrv_ctrl7_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL7_DRV871XX);

                gDeviceSpiReg.idrv_ctrl8_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL8_DRV871XX);

                gDeviceSpiReg.idrv_ctrl9_reg = SPI_readRegister(
                        (uint8_t) IDRV_CTRL9_DRV871XX);

                gDeviceSpiReg.drv_ctrl1_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL1_DRV871XX);

                gDeviceSpiReg.drv_ctrl2_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL2_DRV871XX);

                gDeviceSpiReg.drv_ctrl3_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL3_DRV871XX);

                gDeviceSpiReg.drv_ctrl4_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL4_DRV871XX);

                gDeviceSpiReg.drv_ctrl5_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL5_DRV871XX);

                gDeviceSpiReg.drv_ctrl6_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL6_DRV871XX);

                gDeviceSpiReg.drv_ctrl7_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL7_DRV871XX);

                gDeviceSpiReg.vds_ctrl1_reg = SPI_readRegister(
                        (uint8_t) VDS_CTRL1_DRV871XX);

                gDeviceSpiReg.vds_ctrl2_reg = SPI_readRegister(
                        (uint8_t) VDS_CTRL2_DRV871XX);

                gDeviceSpiReg.vds_ctrl3_reg = SPI_readRegister(
                        (uint8_t) VDS_CTRL3_DRV871XX);

                gDeviceSpiReg.vds_ctrl4_reg = SPI_readRegister(
                        (uint8_t) VDS_CTRL4_DRV871XX);

                gDeviceSpiReg.olsc_ctrl1_reg = SPI_readRegister(
                        (uint8_t) OLSC_CTRL1_DRV871XX);

                gDeviceSpiReg.olsc_ctrl2_reg = SPI_readRegister(
                        (uint8_t) OLSC_CTRL2_DRV871XX);

                gDeviceSpiReg.uvov_ctrl_reg = SPI_readRegister(
                        (uint8_t) UVOV_CTRL_DRV871XX);

                gDeviceSpiReg.csa_ctrl1_reg = SPI_readRegister(
                        (uint8_t) CSA_CTRL1_DRV871XX);

                gDeviceSpiReg.csa_ctrl2_reg = SPI_readRegister(
                        (uint8_t) CSA_CTRL2_DRV871XX);

                gDeviceSpiReg.csa_ctrl3_reg = SPI_readRegister(
                        (uint8_t) CSA_CTRL3_DRV871XX);

                gDeviceSpiReg.agd_ctrl1_reg = SPI_readRegister(
                        (uint8_t) AGD_CTRL1_DRV871XX);

                gDeviceSpiReg.pdr_ctrl1_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL1_DRV871XX);

                gDeviceSpiReg.pdr_ctrl2_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL2_DRV871XX);

                gDeviceSpiReg.pdr_ctrl3_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL3_DRV871XX);

                gDeviceSpiReg.pdr_ctrl4_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL4_DRV871XX);

                gDeviceSpiReg.pdr_ctrl5_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL5_DRV871XX);

                gDeviceSpiReg.pdr_ctrl6_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL6_DRV871XX);

                gDeviceSpiReg.pdr_ctrl7_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL7_DRV871XX);

                gDeviceSpiReg.pdr_ctrl8_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL8_DRV871XX);

                gDeviceSpiReg.pdr_ctrl9_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL9_DRV871XX);

                gDeviceSpiReg.pdr_ctrl10_reg = SPI_readRegister(
                        (uint8_t) PDR_CTRL10_DRV871XX);

                gDeviceSpiReg.stc_ctrl1_reg = SPI_readRegister(
                        (uint8_t) STC_CTRL1_DRV871XX);

                gDeviceSpiReg.stc_ctrl2_reg = SPI_readRegister(
                        (uint8_t) STC_CTRL2_DRV871XX);

                gDeviceSpiReg.stc_ctrl3_reg = SPI_readRegister(
                        (uint8_t) STC_CTRL3_DRV871XX);

                gDeviceSpiReg.stc_ctrl4_reg = SPI_readRegister(
                        (uint8_t) STC_CTRL4_DRV871XX);

                gDeviceSpiReg.dcc_ctrl1_reg = SPI_readRegister(
                        (uint8_t) DCC_CTRL1_DRV871XX);

                gDeviceSpiReg.pst_ctrl1_reg = SPI_readRegister(
                        (uint8_t) PST_CTRL1_DRV871XX);

                gDeviceSpiReg.pst_ctrl2_reg = SPI_readRegister(
                        (uint8_t) PST_CTRL2_DRV871XX);

                gDeviceSpiReg.sgd_stat1_reg = SPI_readRegister(
                        (uint8_t) SGD_STAT1_DRV871XX);

                gDeviceSpiReg.sgd_stat2_reg = SPI_readRegister(
                        (uint8_t) SGD_STAT2_DRV871XX);

                gDeviceSpiReg.sgd_stat3_reg = SPI_readRegister(
                        (uint8_t) SGD_STAT3_DRV871XX);

            }
            else                                                             // Device is DRV8106S-Q1 or DRV8705S-Q1 or DRV8706S -Q1
            {
                gDeviceSpiReg.ic_status_1_reg = SPI_readRegister(
                        (uint8_t) IC_STAT_1);

                gDeviceSpiReg.vgs_vds_stat_reg = SPI_readRegister(
                        (uint8_t) VGS_VDS_STAT);

                gDeviceSpiReg.ic_status_2_reg = SPI_readRegister(
                        (uint8_t) IC_STAT_2);

                gDeviceSpiReg.ic_ctrl_reg = SPI_readRegister(
                        (uint8_t) IC_CTRL);

                gDeviceSpiReg.brg_ctrl_reg = SPI_readRegister(
                        (uint8_t) BRG_CTRL);

                gDeviceSpiReg.drv_ctrl_1_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL_1);

                gDeviceSpiReg.drv_ctrl_2_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL_2);

                gDeviceSpiReg.drv_ctrl_3_reg = SPI_readRegister(
                        (uint8_t) DRV_CTRL_3);

                gDeviceSpiReg.vds_ctrl_1_reg = SPI_readRegister(
                        (uint8_t) VDS_CTRL_1);

                gDeviceSpiReg.vds_ctrl_2_reg = SPI_readRegister(
                        (uint8_t) VDS_CTRL_2);

                gDeviceSpiReg.olsc_ctrl_reg = SPI_readRegister(
                        (uint8_t) OLSC_CTRL);

                gDeviceSpiReg.uvov_ctrl_reg = SPI_readRegister(
                        (uint8_t) UVOV_CTRL);

                gDeviceSpiReg.csa_ctrl_reg = SPI_readRegister(
                        (uint8_t) CSA_CTRL);

            }
            gDRV8x06_Obj.ReadCmd = false;
        }

        if (gDRV8x06_Obj.ManReadCmd)
        {
            // Registers Manual Read using GUI App
            drvRegAddr = gDRV8x06_Obj.ManReadAddr;
            drvDataNew = SPI_readRegister((uint8_t) drvRegAddr);

            gDRV8x06_Obj.ManReadData = drvDataNew;

            if (DRV871x)                                                     // If device is DRV8714S-Q1 or DRV8718S-Q1
            {
                switch (drvRegAddr)
                {

                case IC_STAT_1_DRV871XX :
                    gDeviceSpiReg.ic_stat1_reg = drvDataNew;
                    break;

                case VDS_STAT1_DRV871XX :
                    gDeviceSpiReg.vds_stat1_reg = drvDataNew;
                    break;

                case VDS_STAT2_DRV871XX :
                    gDeviceSpiReg.vds_stat2_reg = drvDataNew;
                    break;

                case VGS_STAT1_DRV871XX :
                    gDeviceSpiReg.vgs_stat1_reg = drvDataNew;
                    break;

                case VGS_STAT2_DRV871XX :
                    gDeviceSpiReg.vgs_stat2_reg = drvDataNew;
                    break;

                case IC_STAT2_DRV871XX :
                    gDeviceSpiReg.ic_stat2_reg = drvDataNew;
                    break;

                case IC_STAT3_DRV871XX :
                    gDeviceSpiReg.ic_stat3_reg = drvDataNew;
                    break;

                case IC_CTRL1_DRV871XX :
                    gDeviceSpiReg.ic_ctrl1_reg = drvDataNew;
                    break;

                case IC_CTRL2_DRV871XX :
                    gDeviceSpiReg.ic_ctrl2_reg = drvDataNew;
                    break;

                case BRG_CTRL1_DRV871XX :
                    gDeviceSpiReg.brg_ctrl1_reg = drvDataNew;
                    break;

                case BRG_CTRL2_DRV871XX :
                    gDeviceSpiReg.brg_ctrl2_reg = drvDataNew;
                    break;

                case PWM_CTRL1_DRV871XX :
                    gDeviceSpiReg.pwm_ctrl1_reg = drvDataNew;
                    break;

                case PWM_CTRL2_DRV871XX :
                    gDeviceSpiReg.pwm_ctrl2_reg = drvDataNew;
                    break;

                case PWM_CTRL3_DRV871XX :
                    gDeviceSpiReg.pwm_ctrl3_reg = drvDataNew;
                    break;

                case PWM_CTRL4_DRV871XX :
                    gDeviceSpiReg.pwm_ctrl4_reg = drvDataNew;
                    break;

                case IDRV_CTRL1_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl1_reg = drvDataNew;
                    break;

                case IDRV_CTRL2_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl2_reg = drvDataNew;
                    break;

                case IDRV_CTRL3_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl3_reg = drvDataNew;
                    break;

                case IDRV_CTRL4_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl4_reg = drvDataNew;
                    break;

                case IDRV_CTRL5_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl5_reg = drvDataNew;
                    break;

                case IDRV_CTRL6_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl6_reg = drvDataNew;
                    break;

                case IDRV_CTRL7_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl7_reg = drvDataNew;
                    break;

                case IDRV_CTRL8_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl8_reg = drvDataNew;
                    break;

                case IDRV_CTRL9_DRV871XX :
                    gDeviceSpiReg.idrv_ctrl9_reg = drvDataNew;
                    break;

                case DRV_CTRL1_DRV871XX :
                    gDeviceSpiReg.drv_ctrl1_reg = drvDataNew;
                    break;

                case DRV_CTRL2_DRV871XX :
                    gDeviceSpiReg.drv_ctrl2_reg = drvDataNew;
                    break;

                case DRV_CTRL3_DRV871XX :
                    gDeviceSpiReg.drv_ctrl3_reg = drvDataNew;
                    break;

                case DRV_CTRL4_DRV871XX :
                    gDeviceSpiReg.drv_ctrl4_reg = drvDataNew;
                    break;

                case DRV_CTRL5_DRV871XX :
                    gDeviceSpiReg.drv_ctrl5_reg = drvDataNew;
                    break;

                case DRV_CTRL6_DRV871XX :
                    gDeviceSpiReg.drv_ctrl6_reg = drvDataNew;
                    break;

                case DRV_CTRL7_DRV871XX :
                    gDeviceSpiReg.drv_ctrl7_reg = drvDataNew;
                    break;

                case VDS_CTRL1_DRV871XX :
                    gDeviceSpiReg.vds_ctrl1_reg = drvDataNew;
                    break;

                case VDS_CTRL2_DRV871XX :
                    gDeviceSpiReg.vds_ctrl2_reg = drvDataNew;
                    break;

                case VDS_CTRL3_DRV871XX :
                    gDeviceSpiReg.vds_ctrl3_reg = drvDataNew;
                    break;

                case VDS_CTRL4_DRV871XX :
                    gDeviceSpiReg.vds_ctrl4_reg = drvDataNew;
                    break;

                case OLSC_CTRL1_DRV871XX :
                    gDeviceSpiReg.olsc_ctrl1_reg = drvDataNew;
                    break;

                case OLSC_CTRL2_DRV871XX :
                    gDeviceSpiReg.olsc_ctrl2_reg = drvDataNew;
                    break;

                case UVOV_CTRL_DRV871XX :
                    gDeviceSpiReg.uvov_ctrl_reg = drvDataNew;
                    break;

                case CSA_CTRL1_DRV871XX :
                    gDeviceSpiReg.csa_ctrl1_reg = drvDataNew;
                    break;

                case CSA_CTRL2_DRV871XX :
                    gDeviceSpiReg.csa_ctrl2_reg = drvDataNew;
                    break;

                case CSA_CTRL3_DRV871XX :
                    gDeviceSpiReg.csa_ctrl3_reg = drvDataNew;
                    break;

                case AGD_CTRL1_DRV871XX :
                    gDeviceSpiReg.agd_ctrl1_reg = drvDataNew;
                    break;

                case PDR_CTRL1_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl1_reg = drvDataNew;
                    break;

                case PDR_CTRL2_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl2_reg = drvDataNew;
                    break;

                case PDR_CTRL3_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl3_reg = drvDataNew;
                    break;

                case PDR_CTRL4_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl4_reg = drvDataNew;
                    break;

                case PDR_CTRL5_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl5_reg = drvDataNew;
                    break;

                case PDR_CTRL6_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl6_reg = drvDataNew;
                    break;

                case PDR_CTRL7_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl7_reg = drvDataNew;
                    break;

                case PDR_CTRL8_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl8_reg = drvDataNew;
                    break;

                case PDR_CTRL9_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl9_reg = drvDataNew;
                    break;

                case PDR_CTRL10_DRV871XX :
                    gDeviceSpiReg.pdr_ctrl10_reg = drvDataNew;
                    break;

                case STC_CTRL1_DRV871XX :
                    gDeviceSpiReg.stc_ctrl1_reg = drvDataNew;
                    break;

                case STC_CTRL2_DRV871XX :
                    gDeviceSpiReg.stc_ctrl2_reg = drvDataNew;
                    break;

                case STC_CTRL3_DRV871XX :
                    gDeviceSpiReg.stc_ctrl3_reg = drvDataNew;
                    break;

                case STC_CTRL4_DRV871XX :
                    gDeviceSpiReg.stc_ctrl4_reg = drvDataNew;
                    break;

                case DCC_CTRL1_DRV871XX :
                    gDeviceSpiReg.dcc_ctrl1_reg = drvDataNew;
                    break;

                case PST_CTRL1_DRV871XX :
                    gDeviceSpiReg.pst_ctrl1_reg = drvDataNew;
                    break;

                case PST_CTRL2_DRV871XX :
                    gDeviceSpiReg.pst_ctrl2_reg = drvDataNew;
                    break;

                case SGD_STAT1_DRV871XX :
                    gDeviceSpiReg.sgd_stat1_reg = drvDataNew;
                    break;

                case SGD_STAT2_DRV871XX :
                    gDeviceSpiReg.sgd_stat2_reg = drvDataNew;
                    break;

                case SGD_STAT3_DRV871XX :
                    gDeviceSpiReg.sgd_stat3_reg = drvDataNew;
                    break;

                default:
                    break;

                } // End of Switch
            }
            else                                                             // Else if device is DRV8106S-Q1 or DRV8705S-Q1 or DRV8706S -Q1
            {
                switch (drvRegAddr)
                {

                case IC_STAT_1 :
                    gDeviceSpiReg.ic_status_1_reg = drvDataNew;
                    break;

                case VGS_VDS_STAT :
                    gDeviceSpiReg.vgs_vds_stat_reg = drvDataNew;
                    break;

                case IC_STAT_2 :
                    gDeviceSpiReg.ic_status_2_reg = drvDataNew;
                    break;

                case IC_CTRL :
                    gDeviceSpiReg.ic_ctrl_reg = drvDataNew;
                    break;

                case BRG_CTRL :
                    gDeviceSpiReg.brg_ctrl_reg = drvDataNew;
                    break;

                case DRV_CTRL_1 :
                    gDeviceSpiReg.drv_ctrl_1_reg = drvDataNew;
                    break;

                case DRV_CTRL_2 :
                    gDeviceSpiReg.drv_ctrl_2_reg = drvDataNew;
                    break;

                case DRV_CTRL_3 :
                    gDeviceSpiReg.drv_ctrl_3_reg = drvDataNew;
                    break;

                case VDS_CTRL_1 :
                    gDeviceSpiReg.vds_ctrl_1_reg = drvDataNew;
                    break;

                case VDS_CTRL_2 :
                    gDeviceSpiReg.vds_ctrl_2_reg = drvDataNew;
                    break;

                case OLSC_CTRL :
                    gDeviceSpiReg.olsc_ctrl_reg = drvDataNew;
                    break;

                case UVOV_CTRL :
                    gDeviceSpiReg.uvov_ctrl_reg = drvDataNew;
                    break;

                case CSA_CTRL :
                    gDeviceSpiReg.csa_ctrl_reg = drvDataNew;
                    break;

                default:
                    break;

                } // End of Switch
            }

            gDRV8x06_Obj.ManReadCmd = false;
        } // End of Read Cmd
    } // End "If device awake"
} // End update_Regs()

void reg_Init(void)
{
    if (DRV871x)                                                             // Device is DRV8714S-Q1 or DRV8718S-Q1
    {
        register_updated = 1;
        SPI_writeRegister(IC_CTRL1_DRV871XX, 0x07);                          // Independent mode, enable driver and clear fault

        SPI_writeRegister(IDRV_CTRL1_DRV871XX, 0xEE);                        // IDRIVE = 48mA/48mA
        SPI_writeRegister(IDRV_CTRL2_DRV871XX, 0xEE);                        // IDRIVE = 48mA/48mA
        SPI_writeRegister(IDRV_CTRL3_DRV871XX, 0xEE);                        // IDRIVE = 48mA/48mA
        SPI_writeRegister(IDRV_CTRL4_DRV871XX, 0xEE);                        // IDRIVE = 48mA/48mA

        SPI_writeRegister(VDS_CTRL1_DRV871XX, 0xAA);                         // VDS_HS_LVL and VDS_LS_LVL = 0.5V
        SPI_writeRegister(VDS_CTRL2_DRV871XX, 0xAA);                         // VDS_HS_LVL and VDS_LS_LVL = 0.5V

        if (DRV8718S == 1)                                                   // Device is DRV8718S-Q1
        {
            SPI_writeRegister(IDRV_CTRL5_DRV871XX, 0xEE);                    // IDRIVE = 48mA/48mA
            SPI_writeRegister(IDRV_CTRL6_DRV871XX, 0xEE);                    // IDRIVE = 48mA/48mA
            SPI_writeRegister(IDRV_CTRL7_DRV871XX, 0xEE);                    // IDRIVE = 48mA/48mA
            SPI_writeRegister(IDRV_CTRL8_DRV871XX, 0xEE);                    // IDRIVE = 48mA/48mA

            SPI_writeRegister(VDS_CTRL3_DRV871XX, 0xAA);                     // VDS_HS_LVL and VDS_LS_LVL = 0.5V
            SPI_writeRegister(VDS_CTRL4_DRV871XX, 0xAA);                     // VDS_HS_LVL and VDS_LS_LVL = 0.5V
        }
    }
    else                                                                     // Device is DRV8106S-Q1 or DRV8705/6S-Q1
    {
        register_updated = 1;
        SPI_writeRegister(IC_CTRL, 0x87);                                    // enable driver and clear fault

        if (gDRV8x06_Obj.deviceID == 3)                                      // Device is DRV8106S-Q1
            SPI_writeRegister(BRG_CTRL, 0x00);                               // half-bridge mode firmware default
        else
            SPI_writeRegister(BRG_CTRL, 0x20);                               // Device is DRV8705/066S-Q1 PH/EN mode firmware default

        SPI_writeRegister(DRV_CTRL_1, 0xEE);                                 // IDRIVE = 48mA/48mA
        SPI_writeRegister(DRV_CTRL_2, 0xEE);                                 // IDRIVE = 48mA/48mA

        SPI_writeRegister(VDS_CTRL_2, 0xAA);                                 // VDS_HS_LVL and VDS_LS_LVL = 0.5V
    }

    gDRV8x06_Obj.ReadCmd = 1;
    update_Regs();
}

/*
 * Function to control the duty cycle for independent half bridge control
 */
void drv_IHBRampControl(void)
{
    drv_RampCTRL(&gBrushed_Obj.rampInput1, gBrushed_Obj.in1En);
    drv_RampCTRL(&gBrushed_Obj.rampInput2, gBrushed_Obj.in2Ph);

    if (DRV871x == 1)                                                        // drv871x-q1
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput3, gBrushed_Obj.in3En);
        drv_RampCTRL(&gBrushed_Obj.rampInput4, gBrushed_Obj.in4Ph);
    }
}

/*
 * Function to control the duty cycle for PH_EN mode
 */
void drv_PH_EN_RampControl(void)
{
    if (gBrushed_Obj.ph == 1 && pre_ph == 0)                                 // Direction control
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput1, 0);                           // en_in1 ramp during direction reversal
        if (gBrushed_Obj.rampInput1.setpointValue == 0)
        {
            pre_ph = 1;
            P4OUT |= IN2_PH;                                                 // forward
        }
    }
    else if (gBrushed_Obj.ph == 0 && pre_ph == 1)                            // Direction control
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput1, 0);                           // en_in1 ramp during direction reversal
        if (gBrushed_Obj.rampInput1.setpointValue == 0)
        {
            pre_ph = 0;
            P4OUT &= ~IN2_PH;                                                // reverse
        }
    }
    else
        /*en_in1 ramp for PH/EN in normal condition*/
        drv_RampCTRL(&gBrushed_Obj.rampInput1, gBrushed_Obj.in1En);

    if (DRV871x == 1)                                                        // drv871x-q1
    {
        if (gBrushed_Obj.ph2 == 1 && pre_ph2 == 0) //Direction control
        {
            drv_RampCTRL(&gBrushed_Obj.rampInput3, 0);                       // en_in3 ramp during direction reversal
            if (gBrushed_Obj.rampInput3.setpointValue == 0)
            {
                pre_ph2 = 1;
                P4OUT |= IN4_PH;                                             // forward
            }
        }
        else if (gBrushed_Obj.ph2 == 0 && pre_ph2 == 1)                      // Direction control
        {
            drv_RampCTRL(&gBrushed_Obj.rampInput3, 0);                       // en_in3 ramp during direction reversal
            if (gBrushed_Obj.rampInput3.setpointValue == 0)
            {
                pre_ph2 = 0;
                P4OUT &= ~IN4_PH;                                            // reverse
            }
        }
        else
            /*en_in1 ramp for PH/EN in normal condition*/
            drv_RampCTRL(&gBrushed_Obj.rampInput3, gBrushed_Obj.in3En);
    }
}

/*
 * Function to ramp down the PWM duty cycle to zero for PH/EN, PWM and Independent bridge modes
 */
void drv_PWM_RampDown(void)
{
    if (gBrushed_Obj.pwmMode == 0)                                           // PH/EN Full-Bridge mode
    {
        if (!DRV871x)                                                        // DRV870xS/H-Q1
        {
            while (TBCCR2 != 0)                                              // Ramp down IN1 PWM to 0
            {
                if (timerUpdate)
                {
                    updatePWM();
                    timerUpdate = 0;
                }
            }
        }
        else                                                                 // DRV8714S/H-Q1
        {
            while (TBCCR2 != 0 && TBCCR5 != 0)                               // Ramp down IN1 PWM and IN3 PWM to 0
            {
                if (timerUpdate)
                {
                    updatePWM();
                    timerUpdate = 0;
                }
            }
        }
    }
    else if (gBrushed_Obj.pwmMode == 1)                                      // Independent Half-Bridge mode
    {
        if (!DRV871x)                                                        // DRV8106S/H-Q1, DRV870xS/H-Q1
        {
            while (TBCCR2 != 0 && TBCCR3 != 0)
            {
                if (timerUpdate)
                {
                    updatePWM();
                    timerUpdate = 0;
                }
            }
        }
        else                                                                 // DRV871xS/H-Q1
        {
            while (TBCCR2 != 0 && TBCCR3 != 0 && TBCCR5 != 0
                    && TBCCR6 != 0)
            {
                if (timerUpdate)
                {
                    updatePWM();
                    timerUpdate = 0;
                }
            }
        }
    }
    else if (gBrushed_Obj.pwmMode == 2)                                      // PWM control Full-Bridge mode
    {
        if (!DRV871x)                                                        // DRV8106S/H-Q1, DRV870xS/H-Q1
        {
            while (TBCCR2 != 800 && TBCCR3 != 800)
            {
                if (timerUpdate)
                {
                    updatePWM();
                    timerUpdate = 0;
                }
            }
        }
        else                                                                 // DRV871xS/H-Q1
        {
            while (TBCCR2 != 800 && TBCCR3 != 800 && TBCCR5 != 800
                    && TBCCR6 != 800)
            {
                if (timerUpdate)
                {
                    updatePWM();
                    timerUpdate = 0;
                }
            }
        }
    }
} // End function to ramp down the PWM duty cycle to zero

/*
 * Function to ramp up and down the PWM duty cycle to target duty cycle value
 */
void drv_RampCTRL(volatile RMPCNTL_t* rmpCntl, uint8_t targetValue)
{
    if (rmpCntl->rampDelayCount++ >= rmpCntl->rampDelayMax)
    {
        if (targetValue > rmpCntl->setpointValue)
            rmpCntl->setpointValue++;
        else if (targetValue < rmpCntl->setpointValue)
            rmpCntl->setpointValue--;

        rmpCntl->rampDelayCount = 0;
    }
} // End function to ramp down the PWM duty cycle to zero

void drv_MCU_GUI_Init()                                                      // Initialize DRV, GUI and MCU
{
    WDTCTL = WDTPW | WDTHOLD;                                                // Stop Watchdog Timer
    clock_Init();                                                            // Setup MSP430 clocks
    uartInit();                                                              // Initialize UART
    gpio_Init();                                                             // Initialize GPIO
    drv_deviceID();                                                          // Read device identification via MSP430 GPIO driven logic combinations
    SetVariantFlags();                                                       // Set a flag for S variants

    //Device specific GPIO settings
    if (DRV871x == 1)                                                        // Device is DRV8714S/H-Q1 or DRV8718S-Q1
        gpio_Init_Drv871x();
    else
        gpio_Init_Drv870x_8106();                                            // Device is DRV8106S/H-Q1 or DRV8705/06S/H-Q1

    __delay_cycles(20000);                                                   // 1.25ms delay
    setDefault_GUI_DRV_Vals();                                               // Setup default GUI App DRV device controls
    pwmTimer_Init();                                                         // Setup Timer B0 CCR0-3
    adc12_Init();                                                            // Setup ADC12
    watchdogTimer_Init();                                                    // Configure Watchdog interval Timer
}
