/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  main.c - c file for main function
 *  DRV87xx_8106-Q1EVM_FIRMWARE
 *  10/25/2021
 *  Murugavel Raju
 *
 ******************************************************************************/
#include <brushed.h>
#include <drv8x06.h>
#include "msp430.h"
#include "uart.h"
#include "Serial_Cmd_Monitor.h"
#include "Spi.h"

extern int pre_ph;
extern int pre_ph2;
extern uint16_t SOSense;
extern uint16_t SOSense2;
uint16_t timerUpdate = 0;
uint16_t SOSense_AREF_Adjusted = 2048;
uint16_t SOSense2_AREF_Adjusted = 2048;
volatile unsigned int SOSense_Raw[NumSamples];
volatile unsigned int SOSense2_Raw[NumSamples];
volatile unsigned char Adcresults = 0;
volatile unsigned char SamplesCollected = 0;
volatile int count = 0;
unsigned int tbccr1_value = TBCCR1_DEFAULT;
unsigned int tbccr1_HB1value = TBCCR1_HB_DEFAULT;
unsigned int tbccr1_HB2value = TBCCR1_HB_DEFAULT;

/***************************************Main***********************************/
int main(void)
{
    drv_MCU_GUI_Init();                        // includes SetVariantFlags
    ClearBufferRelatedParam();                 // Serial protocol between GUI USB and MCU UART
    updateSOSenseAREF();
    _nop();

    while (1)
    {
        updateDeviceControls();                // Read device values and pass to GUI

        if (SVariant && nSLEEP_is_hi)          // Device is S variant and awake
            updateSVariants();                 // Read and update PWM mode from Registers

        updateSOReading();                     // Update SOReading from ADC12 output

        //Start Motor is pressed, ramp up PWM
        if (gBrushed_Obj.startMotor && !gBrushed_Obj.pwmState)
        {
            gBrushed_Obj.pwmState = 1;
            drv_setCtrlMode();                 // Update bridge control mode
        }
        // End Start Motor is pressed, ramp up PWM

        //Stop Motor is pressed, ramp down PWM
        if (!gBrushed_Obj.startMotor && gBrushed_Obj.pwmState)
        {
            gBrushed_Obj.pwmState = 0;
            drv_PWM_RampDown();                // Wait until duty cycles are 0% to continue firmware operation
        }
        // End Stop Motor is pressed, ramp down PWM

        //Update Sense Output values to the GUI
        if ((nSLEEP_is_hi && gDRV8x06_Obj.deviceID == 8 && gDeviceSpiReg.ic_ctrl1_reg & 0x80) ||
                (nSLEEP_is_hi && DRV8718S && gDeviceSpiReg.ic_ctrl1_reg & 0x80) ||
                (nSLEEP_is_hi && gDRV8x06_Obj.deviceID == 9) ||
                (nSLEEP_is_hi && !gDRV8x06_Obj.drvOFF && !DRV871x))
                                               //Check if output drive is enabled and device is not in sleep
        {
            gBrushed_Obj.ADC12read = SOSense;  // Update GUI SO Output 1 display
            gBrushed_Obj.ADC12read2 = SOSense2;// Update GUI SO Output 1 display
        }
        else                                   // If drive is disabled or device is asleep display default values in the GUI
        {
            updateSOSenseAREF();
            gBrushed_Obj.ADC12read = SOSense_AREF_Adjusted;
            gBrushed_Obj.ADC12read2 = SOSense2_AREF_Adjusted;
        }
        // End Update Sense Output values to the GUI

        //PWM duty cycle update
        if (timerUpdate)
        {
            updatePWM();
            timerUpdate = 0;

            // ADC sampling points spread within PWM periods, for low-side CSA spread across active PWM only
            if (gDRV8x06_Obj.deviceID == 5 || gDRV8x06_Obj.deviceID == 6)    // If DRV8705S/H-Q1 low-side CSA
            {
                if (gBrushed_Obj.pwmMode == 0 || gBrushed_Obj.pwmMode == 2)  // Independent half-bridges or PWM control Full-bridge mode
                {
                    tbccr1_HB1value -= 4;                                    // Decrement sampling point
                    tbccr1_HB2value -= 4;                                    // Decrement sampling point

                    if (tbccr1_HB1value < (TBCCR2 + 60))                     // Limit ADC trigger points to IN1 PWM on time
                    {
                        tbccr1_HB1value = TBCCR1_HB_DEFAULT;                 // Restart sliding trigger
                    }

                    if (tbccr1_HB2value < (TBCCR3 + 60))                     // Limit ADC trigger points to !N2 PWM on time
                    {
                        tbccr1_HB2value = TBCCR1_HB_DEFAULT;                 // Restart sliding trigger
                    }

                    if (TBCCR2 < TBCCR3)
                        tbccr1_value = tbccr1_HB1value;
                    else
                        tbccr1_value = tbccr1_HB2value;
                }
                else if (gBrushed_Obj.pwmMode == 1)                          // If in PH/EN full-bridge mode
                {
                    tbccr1_value += 4;                                       // Increment sampling point
                    if (tbccr1_value > TBCCR2 - TBCCR1_DEFAULT)              // Limit ADC trigger points to PWM on time
                        tbccr1_value = TBCCR1_DEFAULT;                       // Restart if value hits TBCCR2 SO output during recirculation in ground sense is zero
                }
            }
            else                                                             // In-line CSA devices
            {
                tbccr1_value += 4;                                           // Increment sampling point in the PWM every cycle
                if (tbccr1_value > 750)                                      // Spread ADC trigger points over entire PWM period for in-line CSA
                    tbccr1_value = TBCCR1_DEFAULT;                           // Restart if value hits 100%
            }

            TBCCR1 = tbccr1_value;                                           // Update CCR1 to trigger ADC12 conversion at a different position in the PWM
            // End Move ADC sampling point across PWM periods
        }
        // End PWM duty cycle update

        __bis_SR_register(GIE);                                              // Enable Interrupts
    }
}
/****************************Interrupt Service Routines*****************************/
/*
 * Timer_B0 ISR: Main PWM ISR, Period= 50us
 */
#pragma vector=TIMER0_B0_VECTOR
__interrupt void Timer0_B0(void)
{
    timerUpdate = 1;
}

/*
 * ADC12 interrupt Service Routine(result event) to read current value SO
 */
#pragma vector=ADC12_VECTOR
__interrupt void ADC12_ISR(void)
{
    SOSense_Raw[Adcresults] = ADC12MEM0;       // SO or SO1 result stored to the array from ADC12MEM0

    if (DRV871x)
    {
        SOSense2_Raw[Adcresults] = ADC12MEM1;  // SO2 result stored to the array from ADC12MEM1
    }

    Adcresults++;                              // Advance array pointer

    if (Adcresults == NumSamples)              // All samples collected?
    {
        ADC12CTL0 &= ~ENC;                     // Disable ADC until average calculation function is executed
        SamplesCollected = 1;                  // Flag to manage 128 samples collection for average calculation
        Adcresults = 0;                        // Reset array pointer
    }
}

/*
 * nFAULT interrupt Service Routine(result event)
 */
#pragma vector=PORT1_VECTOR
__interrupt void Port_1(void)
{
    if (DRV871x == 1)
        P1IFG &= ~DRVOFF_nFLT_INT;             // P1.3 IFG cleared to trip again
    else
        P1IFG &= ~nFAULT_INT;                  // P1.4 IFG cleared to trip again

    gDRV8x06_Obj.nFault = 0;
}

/*
 * Status LED derived from the watchdog timer
 */
#pragma vector=WDT_VECTOR
__interrupt void WatchDog_Timer(void)
{
    if (++count > 256)
    {
        P3OUT ^= STATUS;
        count = 0;
    }

    __bic_SR_register_on_exit(LPM0_bits);      // Clear LPM0 bits
}
